@@ -1447,13 +1447,13 @@ quit:
 
 
 
+static void
+get_timestamp(FILE * const ifp)
+{
 /*
    Since the TIFF DateTime string has no timezone information,
    assume that the camera's clock was set to Universal Time.
  */
-static void
-get_timestamp()
-{
   struct tm t;
   time_t ts;
 
@@ -1481,7 +1481,7 @@ parse_exif(FILE * const ifp, int base)
     save = ftell(ifp);
     fseek (ifp, base+val, SEEK_SET);
     if (tag == 0x9003 || tag == 0x9004)
-      get_timestamp();
+      get_timestamp(ifp);
     if (tag == 0x927c) {
       if (!strncmp(make,"SONY",4))
     data_offset = base+val+len;
@@ -1564,7 +1564,7 @@ parse_tiff_ifd(FILE * const ifp, int base, int level)
       make[0] = 0;
     break;
       case 0x132:           /* DateTime tag */
-    get_timestamp();
+    get_timestamp(ifp);
     break;
       case 0x144:           /* TileOffsets */
     if (level) {
@@ -1721,7 +1721,6 @@ parse_external_jpeg(const char * const ifname)
     char * jfile;
     char * jext;
     char * jname;
-    FILE *ifp;
 
     ext  = strrchr (ifname, '.');
     file = strrchr (ifname, '/');
@@ -1747,11 +1746,13 @@ parse_external_jpeg(const char * const ifname)
             *jext = '0';
         }
     if (strcmp (jname, ifname)) {
-        if ((ifp = fopen (jname, "rb"))) {
+        FILE * ifP;
+        ifP = fopen (jname, "rb");
+        if (ifP) {
             if (verbose)
                 pm_message ("Reading metadata from %s...", jname);
-            parse_tiff(ifp, 12);
-            fclose (ifp);
+            parse_tiff(ifP, 12);
+            fclose (ifP);
         }
     }
     if (!timestamp)
