@@ -52,45 +52,6 @@ static unsigned int pixel_count = 0;
 
 /* Byte-swapping junk. */
 
-union cheat {
-    uint32n l;
-    short s;
-    unsigned char c[4];
-};
-
-
-
-static short
-bs_short(short const s) {
-    union cheat u;
-    unsigned char t;
-
-    u.s = s;
-    t = u.c[0];
-    u.c[0] = u.c[1];
-    u.c[1] = t;
-    return u.s;
-}
-
-
-
-static long
-bs_long(long const l) {
-    union cheat u;
-    unsigned char t;
-
-    u.l = l;
-    t = u.c[0];
-    u.c[0] = u.c[3];
-    u.c[3] = t;
-    t = u.c[1];
-    u.c[1] = u.c[2];
-    u.c[2] = t;
-    return u.l;
-}
-
-
-
 static int
 zero_bits(const unsigned long mask) {
 /*----------------------------------------------------------------------------
@@ -198,19 +159,19 @@ processX10Header(X10WDFileHeader *  const h10P,
 
     if ( h10P->file_version != X10WD_FILE_VERSION ) {
         byte_swap = TRUE;
-        h10P->header_size = bs_long( h10P->header_size );
-        h10P->file_version = bs_long( h10P->file_version );
-        h10P->display_type = bs_long( h10P->display_type );
-        h10P->display_planes = bs_long( h10P->display_planes );
-        h10P->pixmap_format = bs_long( h10P->pixmap_format );
-        h10P->pixmap_width = bs_long( h10P->pixmap_width );
-        h10P->pixmap_height = bs_long( h10P->pixmap_height );
-        h10P->window_width = bs_short( h10P->window_width );
-        h10P->window_height = bs_short( h10P->window_height );
-        h10P->window_x = bs_short( h10P->window_x );
-        h10P->window_y = bs_short( h10P->window_y );
-        h10P->window_bdrwidth = bs_short( h10P->window_bdrwidth );
-        h10P->window_ncolors = bs_short( h10P->window_ncolors );
+        h10P->header_size     = pm_bs_long(h10P->header_size);
+        h10P->file_version    = pm_bs_long(h10P->file_version);
+        h10P->display_type    = pm_bs_long(h10P->display_type);
+        h10P->display_planes  = pm_bs_long(h10P->display_planes);
+        h10P->pixmap_format   = pm_bs_long(h10P->pixmap_format);
+        h10P->pixmap_width    = pm_bs_long(h10P->pixmap_width);
+        h10P->pixmap_height   = pm_bs_long(h10P->pixmap_height);
+        h10P->window_width    = pm_bs_short(h10P->window_width);
+        h10P->window_height   = pm_bs_short(h10P->window_height);
+        h10P->window_x        = pm_bs_short(h10P->window_x);
+        h10P->window_y        = pm_bs_short(h10P->window_y);
+        h10P->window_bdrwidth = pm_bs_short(h10P->window_bdrwidth);
+        h10P->window_ncolors  = pm_bs_short(h10P->window_ncolors);
     } else
         byte_swap = FALSE;
 
@@ -236,9 +197,9 @@ processX10Header(X10WDFileHeader *  const h10P,
             if ( fread( &x10colors[i], sizeof(X10Color), 1, file ) != 1 )
                 pm_error( "couldn't read X10 XWD colormap" );
             if ( byte_swap ) {
-                x10colors[i].red = bs_short( x10colors[i].red );
-                x10colors[i].green = bs_short( x10colors[i].green );
-                x10colors[i].blue = bs_short( x10colors[i].blue );
+                x10colors[i].red   = pm_bs_short(x10colors[i].red);
+                x10colors[i].green = pm_bs_short(x10colors[i].green);
+                x10colors[i].blue  = pm_bs_short(x10colors[i].blue);
             }
             if ( x10colors[i].red != x10colors[i].green ||
                  x10colors[i].green != x10colors[i].blue )
@@ -307,31 +268,31 @@ fixH11ByteOrder(X11WDFileHeader *  const h11P,
     if (h11P->file_version == X11WD_FILE_VERSION) {
         memcpy(h11FixedP, h11P, sizeof(*h11FixedP));
     } else {
-        h11FixedP->header_size = bs_long(h11P->header_size);
-        h11FixedP->file_version = bs_long(h11P->file_version);
-        h11FixedP->pixmap_format = bs_long(h11P->pixmap_format);
-        h11FixedP->pixmap_depth = bs_long(h11P->pixmap_depth);
-        h11FixedP->pixmap_width = bs_long(h11P->pixmap_width);
-        h11FixedP->pixmap_height = bs_long(h11P->pixmap_height);
-        h11FixedP->xoffset = bs_long(h11P->xoffset);
-        h11FixedP->byte_order = bs_long(h11P->byte_order);
-        h11FixedP->bitmap_unit = bs_long(h11P->bitmap_unit);
-        h11FixedP->bitmap_bit_order = bs_long(h11P->bitmap_bit_order);
-        h11FixedP->bitmap_pad = bs_long(h11P->bitmap_pad);
-        h11FixedP->bits_per_pixel = bs_long(h11P->bits_per_pixel);
-        h11FixedP->bytes_per_line = bs_long(h11P->bytes_per_line);
-        h11FixedP->visual_class = bs_long(h11P->visual_class);
-        h11FixedP->red_mask = bs_long(h11P->red_mask);
-        h11FixedP->green_mask = bs_long(h11P->green_mask);
-        h11FixedP->blue_mask = bs_long(h11P->blue_mask);
-        h11FixedP->bits_per_rgb = bs_long(h11P->bits_per_rgb);
-        h11FixedP->colormap_entries = bs_long(h11P->colormap_entries);
-        h11FixedP->ncolors = bs_long(h11P->ncolors);
-        h11FixedP->window_width = bs_long(h11P->window_width);
-        h11FixedP->window_height = bs_long(h11P->window_height);
-        h11FixedP->window_x = bs_long(h11P->window_x);
-        h11FixedP->window_y = bs_long(h11P->window_y);
-        h11FixedP->window_bdrwidth = bs_long(h11P->window_bdrwidth);
+        h11FixedP->header_size      = pm_bs_long(h11P->header_size);
+        h11FixedP->file_version     = pm_bs_long(h11P->file_version);
+        h11FixedP->pixmap_format    = pm_bs_long(h11P->pixmap_format);
+        h11FixedP->pixmap_depth     = pm_bs_long(h11P->pixmap_depth);
+        h11FixedP->pixmap_width     = pm_bs_long(h11P->pixmap_width);
+        h11FixedP->pixmap_height    = pm_bs_long(h11P->pixmap_height);
+        h11FixedP->xoffset          = pm_bs_long(h11P->xoffset);
+        h11FixedP->byte_order       = pm_bs_long(h11P->byte_order);
+        h11FixedP->bitmap_unit      = pm_bs_long(h11P->bitmap_unit);
+        h11FixedP->bitmap_bit_order = pm_bs_long(h11P->bitmap_bit_order);
+        h11FixedP->bitmap_pad       = pm_bs_long(h11P->bitmap_pad);
+        h11FixedP->bits_per_pixel   = pm_bs_long(h11P->bits_per_pixel);
+        h11FixedP->bytes_per_line   = pm_bs_long(h11P->bytes_per_line);
+        h11FixedP->visual_class     = pm_bs_long(h11P->visual_class);
+        h11FixedP->red_mask         = pm_bs_long(h11P->red_mask);
+        h11FixedP->green_mask       = pm_bs_long(h11P->green_mask);
+        h11FixedP->blue_mask        = pm_bs_long(h11P->blue_mask);
+        h11FixedP->bits_per_rgb     = pm_bs_long(h11P->bits_per_rgb);
+        h11FixedP->colormap_entries = pm_bs_long(h11P->colormap_entries);
+        h11FixedP->ncolors          = pm_bs_long(h11P->ncolors);
+        h11FixedP->window_width     = pm_bs_long(h11P->window_width);
+        h11FixedP->window_height    = pm_bs_long(h11P->window_height);
+        h11FixedP->window_x         = pm_bs_long(h11P->window_x);
+        h11FixedP->window_y         = pm_bs_long(h11P->window_y);
+        h11FixedP->window_bdrwidth  = pm_bs_long(h11P->window_bdrwidth);
     }
     *h11FixedPP = h11FixedP;
 }
@@ -357,9 +318,9 @@ readX11Colormap(FILE *      const file,
     if (byteSwap) {
         unsigned int i;
         for (i = 0; i < ncolors; ++i) {
-            x11colors[i].red   = bs_short(x11colors[i].red);
-            x11colors[i].green = bs_short(x11colors[i].green);
-            x11colors[i].blue  = bs_short(x11colors[i].blue);
+            x11colors[i].red   = pm_bs_short(x11colors[i].red);
+            x11colors[i].green = pm_bs_short(x11colors[i].green);
+            x11colors[i].blue  = pm_bs_short(x11colors[i].blue);
         }
     }
     if (debug) {
@@ -675,7 +636,7 @@ getinit(FILE *             const ifP,
         pm_error( "couldn't read XWD file header" );
 
     if (h10P->file_version == X10WD_FILE_VERSION ||
-        bs_long(h10P->file_version) == X10WD_FILE_VERSION) {
+        pm_bs_long(h10P->file_version) == X10WD_FILE_VERSION) {
         
         if (verbose)
             pm_message("Input is X10");
@@ -685,7 +646,7 @@ getinit(FILE *             const ifP,
                          red_maskP, green_maskP, blue_maskP, 
                          byte_orderP, bit_orderP);
     } else if (h11P->file_version == X11WD_FILE_VERSION ||
-               bs_long(h11P->file_version) == X11WD_FILE_VERSION) {
+               pm_bs_long(h11P->file_version) == X11WD_FILE_VERSION) {
         
         int rc;
 
@@ -832,8 +793,7 @@ readItem(pixelReader * const rdrP) {
 /*----------------------------------------------------------------------------
    Read one item from the XWD raster associated with pixel reader *rdrP.
 
-   Put the item into the item buffer of the pixel reader object *rdrP
-   with the results.
+   Put the item into the item buffer of the pixel reader object *rdrP.
 -----------------------------------------------------------------------------*/
     assert(rdrP->nBitsLeft == 0);
 
@@ -988,7 +948,7 @@ getpix(pixelReader * const rdrP) {
                     (rdrP->itemBuffer >> nBitsToLeave) & bitsToTakeMask;
             } else {
                 bitsToTake = rdrP->itemBuffer & bitsToTakeMask;
-                bitsToTake >>= nBitsToTake;
+                rdrP->itemBuffer >>= nBitsToTake;
             }                
             /* Shift the bits into the right end of the accumulator */
             pixel <<= nBitsToTake;
