@@ -18,15 +18,6 @@
   contributing their work to the public domain.
 ===========================================================================*/
 
-/* There is some indication that a valid fax always has 1728 columns
-   in every line.  There was some code until Netpbm 10.24 (August 2004)
-   to issue a warning for lines shorter than that, but the code was
-   disabled.  
-
-   The code you see tolerates variable length lines, even within the same
-   fax.  It uses the maximum line size as the PBM width.
-*/
-
 #include "pbm.h"
 #include "shhopt.h"
 #include "nstring.h"
@@ -53,11 +44,12 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *inputFilespec;  /* Filespec of input file */
+    const char * inputFilespec;  /* Filespec of input file */
     unsigned int reversebits;
     unsigned int kludge;
     unsigned int stretch;
     unsigned int stop_error;
+    unsigned int expectedLineSize;
 };
 
 
@@ -69,12 +61,17 @@ parseCommandLine(int argc, char ** const argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry option_def[100];
-    /* Instructions to OptParseOptions2 on how to parse our options.  */
+    optEntry * option_def;  /* malloc'ed */
+        /* Instructions to OptParseOptions3 on how to parse our options.  */
     optStruct3 opt;
 
     unsigned int option_def_index;
 
+    unsigned int widthSpec, paper_sizeSpec;
+    const char * paperSize;
+
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "reversebits",      OPT_FLAG,  NULL, &cmdlineP->reversebits,
             0);
@@ -84,6 +81,10 @@ parseCommandLine(int argc, char ** const argv,
             0);
     OPTENT3(0, "stop_error",       OPT_FLAG,  NULL, &cmdlineP->stop_error, 
             0);
+    OPTENT3(0, "width",            OPT_UINT,  &cmdlineP->expectedLineSize,
+            &widthSpec,                0);
+    OPTENT3(0, "paper_size",       OPT_STRING, &paperSize,
+            &paper_sizeSpec,           0);
     
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -92,6 +93,30 @@ parseCommandLine(int argc, char ** const argv,
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
+    if (widthSpec && paper_sizeSpec)
+        pm_error("You can't specify both -width and -paper_size");
+
+    if (widthSpec) {
+        if (cmdlineP->expectedLineSize < 1)
+            pm_error("-width must be at least 1");
+    } else if (paper_sizeSpec) {
+        if (STRCASEEQ(paperSize, "A6"))
+            cmdlineP->expectedLineSize = 864;
+        else if (STRCASEEQ(paperSize, "A5"))
+            cmdlineP->expectedLineSize = 1216;
+        else if (STRCASEEQ(paperSize, "A4"))
+            cmdlineP->expectedLineSize = 1728;
+        else if (STRCASEEQ(paperSize, "B4"))
+            cmdlineP->expectedLineSize = 2048;
+        else if (STRCASEEQ(paperSize, "A3"))
+            cmdlineP->expectedLineSize = 2432;
+        else
+            pm_error("Unrecognized value for -paper_size '%s'.  "
+                     "We recognize only A3, A4, A5, A6, and B3.",
+                     paperSize);
+    } else
+        cmdlineP->expectedLineSize = 0;
+
     if (argc-1 == 0) 
         cmdlineP->inputFilespec = "-";
     else if (argc-1 != 1)
@@ -389,7 +414,7 @@ readFaxRow(struct bitStream * const bitStreamP,
            const char **      const errorP) {
 /*----------------------------------------------------------------------------
   Read one line of G3 fax from the bit stream *bitStreamP into 
-  bitrow[].  Return the length of the row, in columns, as *lineLengthP.
+  bitrow[].  Return the length of the line, in pixels, as *lineLengthP.
 
   If there's a problem with the line, return as much of it as we can,
   advance the input stream past the next EOL mark, and put a text
@@ -526,6 +551,77 @@ handleRowException(const char * const exception,
 
 
 
+typedef struct {
+    unsigned int expectedLineSize;
+        /* The size that lines are supposed to be.  Zero means we're happy
+           with any size.
+        */
+    unsigned int maxLineSize;
+        /* The maximum line size we have seen so far, or zero if we have
+           not seen any lines yet.
+        */
+    bool warned;
+        /* We have warned the user that he has a line length problem */
+    bool tolerateErrors;
+        /* Try to continue when we detect a line size error, as opposed to
+           aborting the program.
+        */
+} lineSizeAnalyzer;
+
+
+
+static void
+initializeLineSizeAnalyzer(lineSizeAnalyzer * const analyzerP,
+                           unsigned int       const expectedLineSize,
+                           bool               const tolerateErrors) {
+
+    analyzerP->expectedLineSize = expectedLineSize;
+    analyzerP->tolerateErrors   = tolerateErrors;
+
+    analyzerP->maxLineSize = 0;
+    analyzerP->warned      = FALSE;
+}
+
+
+
+static void
+analyzeLineSize(lineSizeAnalyzer * const analyzerP,
+                unsigned int       const thisLineSize) {
+
+    const char * error;
+
+    if (analyzerP->expectedLineSize &&
+        thisLineSize != analyzerP->expectedLineSize)
+        asprintfN(&error, "Image contains a line of %u pixels.  "
+                  "You specified lines should be %u pixels.",
+                  thisLineSize, analyzerP->expectedLineSize);
+    else {
+        if (analyzerP->maxLineSize && thisLineSize != analyzerP->maxLineSize)
+            asprintfN(&error, "There are at least two different "
+                      "line lengths in this image, "
+                      "%u pixels and %u pixels.  "
+                      "This is a violation of the G3 standard.  ",
+                      thisLineSize, analyzerP->maxLineSize);
+        else
+            error = NULL;
+    }
+
+    if (error) {
+        if (analyzerP->tolerateErrors) {
+            if (!analyzerP->warned) {
+                pm_message("Warning: %s.", error);
+                analyzerP->warned = TRUE;
+            }
+        } else
+            pm_error("%s", error);
+
+        strfree(error);
+    }
+    analyzerP->maxLineSize = MAX(thisLineSize, analyzerP->maxLineSize);
+}
+
+
+
 /* An empty line means EOF.  An ancient comment in the code said there
    is supposed to 6 EOL marks in a row to indicate EOF, but the code
    checked for 3 and considered 2 in row just to mean a zero length
@@ -538,20 +634,22 @@ handleRowException(const char * const exception,
 static void
 readFax(struct bitStream * const bitStreamP,
         bool               const stretch,
+        unsigned int       const expectedLineSize,
         bool               const tolerateErrors,
         bit ***            const bitsP,
         unsigned int *     const colsP,
         unsigned int *     const rowsP) {
 
-    unsigned int maxLineSize;
+    lineSizeAnalyzer lineSizeAnalyzer;
     bit ** bits;
     const char * error;
     bool eof;
     unsigned int row;
     
     MALLOCARRAY_NOFAIL(bits, MAXROWS);
-    
-    maxLineSize = 0;  /* initial assumption */
+
+    initializeLineSizeAnalyzer(&lineSizeAnalyzer,
+                               expectedLineSize, tolerateErrors);
 
     eof = FALSE;
     error = NULL;
@@ -571,28 +669,29 @@ readFax(struct bitStream * const bitStreamP,
 
             handleRowException(exception, error, row, tolerateErrors);
 
-            if (lineSize == 0) {
-                /* EOF.  See explanation above */
-                eof = TRUE;
-            } else {
-                maxLineSize = MAX(maxLineSize, lineSize);
+            if (!error) {
+                if (lineSize == 0) {
+                    /* EOF.  See explanation above */
+                    eof = TRUE;
+                } else {
+                    analyzeLineSize(&lineSizeAnalyzer, lineSize);
                     
-                if (stretch) {
+                    if (stretch) {
+                        ++row;
+                        if (row >= MAXROWS)
+                            asprintfN(&error, "Image is too tall.  This "
+                                      "program can handle at most %u rows "
+                                      "after stretching", MAXROWS);
+                        else
+                            bits[row] = bits[row-1];
+                    }
                     ++row;
-                    if (row >= MAXROWS)
-                        asprintfN(&error, "Image is too tall.  This "
-                                  "program can handle at most %u rows "
-                                  "after stretching", MAXROWS);
-                    else
-                        bits[row] = bits[row-1];
                 }
-                ++row;
             }
         }
     }
-
     *rowsP  = row;
-    *colsP  = maxLineSize;
+    *colsP  = lineSizeAnalyzer.maxLineSize;
     *bitsP  = bits;
 }
 
@@ -625,7 +724,8 @@ main(int argc, char * argv[]) {
 
     buildHashes(&whash, &bhash);
 
-    readFax(&bitStream, cmdline.stretch, !cmdline.stop_error, 
+    readFax(&bitStream, cmdline.stretch, cmdline.expectedLineSize,
+            !cmdline.stop_error, 
             &bits, &cols, &rows);
 
     pm_close(ifP);
