@@ -21,7 +21,7 @@
  *  - "-stdpalette" option to enforce this
  *  - row-by-row operation (PPM output)
  *  11/Dec/94:
- *  - support for 24bit and 32bit (24bit + 8bit intensity) images
+ *  - capability for 24bit and 32bit (24bit + 8bit intensity) images
  *  - row-by-row operation (PCX input, for 16-color and truecolor images)
  *  - some code restructuring
  *  15/Feb/95:
@@ -40,16 +40,6 @@
 
 #define PCX_MAXVAL      (pixval)255
 
-/* prototypes */
-static void pcx_16col_to_ppm ARGS((FILE *ifp, int cols, int rows,
-                int BytesPerLine, int BitsPerPixel, int Planes, pixel *cmap));
-static void pcx_256col_to_ppm ARGS((FILE *ifp, int cols, int rows, int BytesPerLine));
-static void pcx_truecol_to_ppm ARGS((FILE *ifp, int cols, int rows, int BytesPerLine, int Planes));
-static void GetPCXRow ARGS((FILE *ifp, unsigned char *pcxrow, int bytesperline));
-static void pcx_planes_to_pixels ARGS(( unsigned char *pixels, unsigned char *bitplanes, int bytesperline, int planes, int bitsperpixel ));
-static void pcx_unpack_pixels ARGS(( unsigned char *pixels, unsigned char *bitplanes, int bytesperline, int planes, int bitsperpixel ));
-static int GetByte ARGS(( FILE *fp ));
-
 /* standard palette */
 static unsigned char const StdRed[]   = { 0, 255,   0,   0, 170, 170, 170, 170, 85,  85,  85,  85, 255, 255, 255, 255 };
 static unsigned char const StdGreen[] = { 0, 255, 170, 170,   0,   0, 170, 170, 85,  85, 255, 255,  85,  85, 255, 255 };
@@ -164,6 +154,19 @@ readInt(FILE * const ifP, short * const retvalP) {
 
 
 
+static int
+GetByte(FILE * const fp) {
+
+    int    c;
+
+    if ((c = fgetc(fp)) == EOF)
+        pm_error("unexpected end of file" );
+
+    return c;
+}
+
+
+
 static void
 readPcxHeader(FILE *             const ifP, 
               struct pcxHeader * const pcxHeaderP) {
@@ -273,77 +276,163 @@ allBlackPalette(pixel cmap16[]) {
 
 
 
-int
-main(int argc, char *argv[]) {
+/*
+ *  read a single row encoded row, handles encoding across rows
+ */
+static void
+GetPCXRow(FILE * const ifP, unsigned char * const pcxrow, 
+          int const bytesperline) {
+/*----------------------------------------------------------------------------
+   Read one row from the PCX raster.
+   
+   The PCX raster is run length encoded as follows:  If the upper two
+   bits of a byte are 11, the lower 6 bits are a repetition count for the
+   raster byte that follows.  If the upper two bits are not 11, the byte 
+   _is_ a raster byte, with repetition count 1.
 
-    FILE *ifP;
-    struct cmdlineInfo cmdline;
-    struct pcxHeader pcxHeader;
-    int Width, Height;
-    pixel *cmap16;
+   A run can't span rows, but it can span planes within a row.  That's
+   why 'repetitionsLeft' and 'c' are static variables in this
+   subroutine.
+-----------------------------------------------------------------------------*/
+    static int repetitionsLeft = 0;
+    static int c;
+    int bytesGenerated;
 
-    ppm_init( &argc, argv );
-
-    generateStdPalette();
-
-    parseCommandLine(argc, argv, &cmdline);
-
-    ifP = pm_openr(cmdline.inputFilespec);
-
-    readPcxHeader(ifP, &pcxHeader);
-
-    if (cmdline.verbose)
-        reportPcxHeader(pcxHeader);
-
-    Width  = (pcxHeader.Xmax - pcxHeader.Xmin) + 1;
-    Height = (pcxHeader.Ymax - pcxHeader.Ymin) + 1;
-
-    if (cmdline.stdpalette || allBlackPalette(pcxHeader.cmap16))
-        cmap16 = stdPalette;
-    else
-        cmap16 = pcxHeader.cmap16;
-
-    ppm_writeppminit(stdout, Width, Height, PCX_MAXVAL, 0);
-    switch (pcxHeader.BitsPerPixel) {
-    case 1:
-        if(pcxHeader.Planes >= 1 && pcxHeader.Planes <= 4)
-            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
-                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
-        else
-            goto fail;
-        break;
-    case 2:
-    case 4:
-        if (pcxHeader.Planes == 1)
-            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
-                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
-        else
-            goto fail;
-        break;
-    case 8:
-        switch(pcxHeader.Planes) {
-        case 1:
-            pcx_256col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine);
-            break;
-        case 3:
-        case 4:
-            pcx_truecol_to_ppm(ifP, Width, Height, 
-                               pcxHeader.BytesPerLine, pcxHeader.Planes);
-            break;
-        default:
-            goto fail;
+    bytesGenerated = 0;
+    while( bytesGenerated < bytesperline ) {
+        if(repetitionsLeft > 0) {
+            pcxrow[bytesGenerated++] = c;
+            --repetitionsLeft;
+        } else {
+            c = GetByte(ifP);
+            if ((c & 0xc0) != 0xc0)
+                /* This is a 1-shot byte, not a repetition count */
+                pcxrow[bytesGenerated++] = c;
+            else {
+                /* This is a repetition count for the following byte */
+                repetitionsLeft = c & 0x3f;
+                c = GetByte(ifP);
+            }
         }
-        break;
-    default:
-    fail:
-        pm_error("can't handle %d bits per pixel image with %d planes",
-                 pcxHeader.BitsPerPixel, pcxHeader.Planes);
     }
-    pm_close(ifP);
-    exit(0);
 }
 
 
+
+/*
+ * convert packed pixel format into 1 pixel per byte
+ */
+static void
+pcx_unpack_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
+    unsigned char   *pixels;
+    unsigned char   *bitplanes;
+    int             bytesperline;
+    int             planes;
+    int             bitsperpixel;
+{
+    register int        bits;
+
+    if (planes != 1)
+        pm_error("can't handle packed pixels with more than 1 plane" );
+#if 0
+    if (bitsperpixel == 8)
+    {
+        while (--bytesperline >= 0)
+            *pixels++ = *bitplanes++;
+    }
+    else
+#endif
+        if (bitsperpixel == 4)
+        {
+            while (--bytesperline >= 0)
+            {
+                bits        = *bitplanes++;
+                *pixels++   = (bits >> 4) & 0x0f;
+                *pixels++   = (bits     ) & 0x0f;
+            }
+        }
+        else if (bitsperpixel == 2)
+        {
+            while (--bytesperline >= 0)
+            {
+                bits        = *bitplanes++;
+                *pixels++   = (bits >> 6) & 0x03;
+                *pixels++   = (bits >> 4) & 0x03;
+                *pixels++   = (bits >> 2) & 0x03;
+                *pixels++   = (bits     ) & 0x03;
+            }
+        }
+        else if (bitsperpixel == 1)
+        {
+            while (--bytesperline >= 0)
+            {
+                bits        = *bitplanes++;
+                *pixels++   = ((bits & 0x80) != 0);
+                *pixels++   = ((bits & 0x40) != 0);
+                *pixels++   = ((bits & 0x20) != 0);
+                *pixels++   = ((bits & 0x10) != 0);
+                *pixels++   = ((bits & 0x08) != 0);
+                *pixels++   = ((bits & 0x04) != 0);
+                *pixels++   = ((bits & 0x02) != 0);
+                *pixels++   = ((bits & 0x01) != 0);
+            }
+        }
+        else
+            pm_error("pcx_unpack_pixels - can't handle %d bits per pixel", 
+                     bitsperpixel);
+}
+
+
+
+/*
+ * convert multi-plane format into 1 pixel per byte
+ */
+static void
+pcx_planes_to_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
+    unsigned char   *pixels;
+    unsigned char   *bitplanes;
+    int             bytesperline;
+    int             planes;
+    int             bitsperpixel;
+{
+    int  i, j;
+    int  npixels;
+    unsigned char    *p;
+
+    if (planes > 4)
+        pm_error("can't handle more than 4 planes" );
+    if (bitsperpixel != 1)
+        pm_error("can't handle more than 1 bit per pixel" );
+
+    /*
+     * clear the pixel buffer
+     */
+    npixels = (bytesperline * 8) / bitsperpixel;
+    p    = pixels;
+    while (--npixels >= 0)
+        *p++ = 0;
+
+    /*
+     * do the format conversion
+     */
+    for (i = 0; i < planes; i++)
+    {
+        int pixbit, bits, mask;
+
+        p    = pixels;
+        pixbit    = (1 << i);
+        for (j = 0; j < bytesperline; j++)
+        {
+            bits = *bitplanes++;
+            for (mask = 0x80; mask != 0; mask >>= 1, p++)
+                if (bits & mask)
+                    *p |= pixbit;
+        }
+    }
+}
+
+
+
 static void
 pcx_16col_to_ppm(ifp, cols, rows, BytesPerLine, BitsPerPixel, Planes, cmap)
     FILE *ifp;
@@ -555,169 +644,75 @@ pcx_truecol_to_ppm(ifp, cols, rows, BytesPerLine, Planes)
 }
 
 
-/*
- * convert multi-plane format into 1 pixel per byte
- */
-static void
-pcx_planes_to_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
-    unsigned char   *pixels;
-    unsigned char   *bitplanes;
-    int             bytesperline;
-    int             planes;
-    int             bitsperpixel;
-{
-    int  i, j;
-    int  npixels;
-    unsigned char    *p;
+int
+main(int argc, char *argv[]) {
 
-    if (planes > 4)
-        pm_error("can't handle more than 4 planes" );
-    if (bitsperpixel != 1)
-        pm_error("can't handle more than 1 bit per pixel" );
+    FILE *ifP;
+    struct cmdlineInfo cmdline;
+    struct pcxHeader pcxHeader;
+    int Width, Height;
+    pixel *cmap16;
 
-    /*
-     * clear the pixel buffer
-     */
-    npixels = (bytesperline * 8) / bitsperpixel;
-    p    = pixels;
-    while (--npixels >= 0)
-        *p++ = 0;
+    ppm_init( &argc, argv );
 
-    /*
-     * do the format conversion
-     */
-    for (i = 0; i < planes; i++)
-    {
-        int pixbit, bits, mask;
+    generateStdPalette();
 
-        p    = pixels;
-        pixbit    = (1 << i);
-        for (j = 0; j < bytesperline; j++)
-        {
-            bits = *bitplanes++;
-            for (mask = 0x80; mask != 0; mask >>= 1, p++)
-                if (bits & mask)
-                    *p |= pixbit;
-        }
-    }
-}
+    parseCommandLine(argc, argv, &cmdline);
 
-/*
- * convert packed pixel format into 1 pixel per byte
- */
-static void
-pcx_unpack_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
-    unsigned char   *pixels;
-    unsigned char   *bitplanes;
-    int             bytesperline;
-    int             planes;
-    int             bitsperpixel;
-{
-    register int        bits;
+    ifP = pm_openr(cmdline.inputFilespec);
 
-    if (planes != 1)
-        pm_error("can't handle packed pixels with more than 1 plane" );
-#if 0
-    if (bitsperpixel == 8)
-    {
-        while (--bytesperline >= 0)
-            *pixels++ = *bitplanes++;
-    }
+    readPcxHeader(ifP, &pcxHeader);
+
+    if (cmdline.verbose)
+        reportPcxHeader(pcxHeader);
+
+    Width  = (pcxHeader.Xmax - pcxHeader.Xmin) + 1;
+    Height = (pcxHeader.Ymax - pcxHeader.Ymin) + 1;
+
+    if (cmdline.stdpalette || allBlackPalette(pcxHeader.cmap16))
+        cmap16 = stdPalette;
     else
-#endif
-        if (bitsperpixel == 4)
-        {
-            while (--bytesperline >= 0)
-            {
-                bits        = *bitplanes++;
-                *pixels++   = (bits >> 4) & 0x0f;
-                *pixels++   = (bits     ) & 0x0f;
-            }
-        }
-        else if (bitsperpixel == 2)
-        {
-            while (--bytesperline >= 0)
-            {
-                bits        = *bitplanes++;
-                *pixels++   = (bits >> 6) & 0x03;
-                *pixels++   = (bits >> 4) & 0x03;
-                *pixels++   = (bits >> 2) & 0x03;
-                *pixels++   = (bits     ) & 0x03;
-            }
-        }
-        else if (bitsperpixel == 1)
-        {
-            while (--bytesperline >= 0)
-            {
-                bits        = *bitplanes++;
-                *pixels++   = ((bits & 0x80) != 0);
-                *pixels++   = ((bits & 0x40) != 0);
-                *pixels++   = ((bits & 0x20) != 0);
-                *pixels++   = ((bits & 0x10) != 0);
-                *pixels++   = ((bits & 0x08) != 0);
-                *pixels++   = ((bits & 0x04) != 0);
-                *pixels++   = ((bits & 0x02) != 0);
-                *pixels++   = ((bits & 0x01) != 0);
-            }
-        }
+        cmap16 = pcxHeader.cmap16;
+
+    ppm_writeppminit(stdout, Width, Height, PCX_MAXVAL, 0);
+    switch (pcxHeader.BitsPerPixel) {
+    case 1:
+        if(pcxHeader.Planes >= 1 && pcxHeader.Planes <= 4)
+            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
+                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
         else
-            pm_error("pcx_unpack_pixels - can't handle %d bits per pixel", 
-                     bitsperpixel);
-}
-
-
-
-static int
-GetByte(FILE * const fp) {
-
-    int    c;
-
-    if ((c = fgetc(fp)) == EOF)
-        pm_error("unexpected end of file" );
-
-    return c;
-}
-
-
-
-/*
- *  read a single row encoded row, handles encoding across rows
- */
-static void
-GetPCXRow(FILE * const ifP, unsigned char * const pcxrow, 
-          int const bytesperline) {
-/*----------------------------------------------------------------------------
-   Read one row from the PCX raster.
-   
-   The PCX raster is run length encoded as follows:  If the upper two
-   bits of a byte are 11, the lower 6 bits are a repetition count for the
-   raster byte that follows.  If the upper two bits are not 11, the byte 
-   _is_ a raster byte, with repetition count 1.
-
-   A run can't span rows, but it can span planes within a row.  That's
-   why 'repetitionsLeft' and 'c' are static variables in this
-   subroutine.
------------------------------------------------------------------------------*/
-    static int repetitionsLeft = 0;
-    static int c;
-    int bytesGenerated;
-
-    bytesGenerated = 0;
-    while( bytesGenerated < bytesperline ) {
-        if(repetitionsLeft > 0) {
-            pcxrow[bytesGenerated++] = c;
-            --repetitionsLeft;
-        } else {
-            c = GetByte(ifP);
-            if ((c & 0xc0) != 0xc0)
-                /* This is a 1-shot byte, not a repetition count */
-                pcxrow[bytesGenerated++] = c;
-            else {
-                /* This is a repetition count for the following byte */
-                repetitionsLeft = c & 0x3f;
-                c = GetByte(ifP);
-            }
+            goto fail;
+        break;
+    case 2:
+    case 4:
+        if (pcxHeader.Planes == 1)
+            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
+                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
+        else
+            goto fail;
+        break;
+    case 8:
+        switch(pcxHeader.Planes) {
+        case 1:
+            pcx_256col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine);
+            break;
+        case 3:
+        case 4:
+            pcx_truecol_to_ppm(ifP, Width, Height, 
+                               pcxHeader.BytesPerLine, pcxHeader.Planes);
+            break;
+        default:
+            goto fail;
         }
+        break;
+    default:
+    fail:
+        pm_error("can't handle %d bits per pixel image with %d planes",
+                 pcxHeader.BitsPerPixel, pcxHeader.Planes);
     }
+    pm_close(ifP);
+    
+    return 0;
 }
 
+
