@@ -34,17 +34,22 @@
    only because they had two different lineages and the code is too
    complicated for me quickly to rewrite it.
 */
+#include <assert.h>
 #include <string.h>
 
-#include "ppm.h"
+#include "mallocvar.h"
 #include "shhopt.h"
+#include "ppm.h"
 
 #define MAXCMAPSIZE 256
 
+static unsigned int const gifMaxval = 255;
+
+static bool verbose;
 /*
  * a code_int must be able to hold 2**BITS values of type int, and also -1
  */
-typedef int             code_int;
+typedef int code_int;
 
 #ifdef SIGNED_COMPARE_SLOW
 typedef unsigned long int count_int;
@@ -97,7 +102,7 @@ struct gif_dest {
     int code_counter;   /* counts output symbols */
 };
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
@@ -110,6 +115,7 @@ struct cmdline_info {
     const char *transparent;    /* -transparent option value.  NULL if none. */
     const char *comment;        /* -comment option value; NULL if none */
     unsigned int nolzw;    /* -nolzw option */
+    unsigned int verbose;
 };
 
 
@@ -142,8 +148,8 @@ handleLatex2htmlHack(void) {
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Parse the program arguments (given by argc and argv) into a form
    the program can deal with more easily -- a cmdline_info structure.
@@ -153,38 +159,42 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
+    optEntry * option_def;  /* malloc'ed */
     optStruct3 opt;  /* set by OPTENT3 */
-    optEntry *option_def = malloc(100*sizeof(optEntry));
     unsigned int option_def_index;
 
     unsigned int latex2htmlhack;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "interlace",   OPT_FLAG,   
-            NULL,                       &cmdline_p->interlace, 0);
+            NULL,                       &cmdlineP->interlace, 0);
     OPTENT3(0,   "sort",        OPT_FLAG,   
-            NULL,                       &cmdline_p->sort, 0);
+            NULL,                       &cmdlineP->sort, 0);
     OPTENT3(0,   "nolzw",       OPT_FLAG,   
-            NULL,                       &cmdline_p->nolzw, 0);
+            NULL,                       &cmdlineP->nolzw, 0);
     OPTENT3(0,   "mapfile",     OPT_STRING, 
-            &cmdline_p->mapfile,        NULL, 0);
+            &cmdlineP->mapfile,        NULL, 0);
     OPTENT3(0,   "transparent", OPT_STRING, 
-            &cmdline_p->transparent,    NULL, 0);
+            &cmdlineP->transparent,    NULL, 0);
     OPTENT3(0,   "comment",     OPT_STRING, 
-            &cmdline_p->comment,        NULL, 0);
+            &cmdlineP->comment,        NULL, 0);
     OPTENT3(0,   "alpha",       OPT_STRING, 
-            &cmdline_p->alpha_filespec, NULL, 0);
+            &cmdlineP->alpha_filespec, NULL, 0);
     OPTENT3(0,   "alphacolor",  OPT_STRING, 
-            &cmdline_p->alphacolor,     NULL, 0);
+            &cmdlineP->alphacolor,     NULL, 0);
     OPTENT3(0,   "h",           OPT_FLAG, 
             NULL,                       &latex2htmlhack, 0);
+    OPTENT3(0,   "verbose",     OPT_FLAG, 
+            NULL,                       &cmdlineP->verbose, 0);
     
     /* Set the defaults */
-    cmdline_p->mapfile = NULL;
-    cmdline_p->transparent = NULL;  /* no transparency */
-    cmdline_p->comment = NULL;      /* no comment */
-    cmdline_p->alpha_filespec = NULL;      /* no alpha file */
-    cmdline_p->alphacolor = "rgb:0/0/0";      
+    cmdlineP->mapfile = NULL;
+    cmdlineP->transparent = NULL;  /* no transparency */
+    cmdlineP->comment = NULL;      /* no comment */
+    cmdlineP->alpha_filespec = NULL;      /* no alpha file */
+    cmdlineP->alphacolor = "rgb:0/0/0";      
         /* We could say "black" here, but then we depend on the color names
            database existing.
         */
@@ -194,30 +204,25 @@ parse_command_line(int argc, char ** argv,
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
-        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (latex2htmlhack) 
         handleLatex2htmlHack();
 
     if (argc-1 == 0) 
-        cmdline_p->input_filespec = "-";
+        cmdlineP->input_filespec = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdline_p->input_filespec = argv[1];
+        cmdlineP->input_filespec = argv[1];
 
-    if (cmdline_p->alpha_filespec && cmdline_p->transparent)
+    if (cmdlineP->alpha_filespec && cmdlineP->transparent)
         pm_error("You cannot specify both -alpha and -transparent.");
 }
 
 
 
-static int __inline__ sqr(const int x) {
-    return x*x;
-}
-
-
 /*
  * Write out a word to the GIF file
  */
@@ -229,8 +234,10 @@ Putword(int const w, FILE * const fp) {
 }
 
 
-static int __inline__
-closestcolor(const pixel color, pixval const maxval, struct cmap * cmapP) {
+static int
+closestcolor(pixel         const color,
+             pixval        const maxval,
+             struct cmap * const cmapP) {
 /*----------------------------------------------------------------------------
    Return the pre-sort colormap index of the color in the colormap *cmapP
    that is closest to the color 'color', whose maxval is 'maxval'.
@@ -239,125 +246,198 @@ closestcolor(const pixel color, pixval const maxval, struct cmap * cmapP) {
    are returning.  Caller must ensure that the color is not already in
    there.
 -----------------------------------------------------------------------------*/
-    int i,r,g,b;
-    int imin,dmin;
+    unsigned int i;
+    unsigned int imin, dmin;
 
-    r=(int)PPM_GETR(color)*255/maxval;
-    g=(int)PPM_GETG(color)*255/maxval;
-    b=(int)PPM_GETB(color)*255/maxval;
+    pixval const r = PPM_GETR(color) * gifMaxval / maxval;
+    pixval const g = PPM_GETG(color) * gifMaxval / maxval;
+    pixval const b = PPM_GETB(color) * gifMaxval / maxval;
 
-    dmin = sqr(255)*3;
+    dmin = SQR(255) * 3;
     imin = 0;
     for (i=0;i < cmapP->cmapsize; i++) {
-        int d;
-        d=sqr(r-cmapP->red[i])+sqr(g-cmapP->green[i])+sqr(b-cmapP->blue[i]);
-        if (d<dmin) {
-            dmin=d;
-            imin=i; 
+        int const d = SQR(r-cmapP->red[i]) + 
+            SQR(g-cmapP->green[i]) + 
+            SQR(b-cmapP->blue[i]);
+        if (d < dmin) {
+            dmin = d;
+            imin = i; 
         } 
     }
-    ppm_addtocolorhash(cmapP->cht,&color,cmapP->permi[imin]);
+    ppm_addtocolorhash(cmapP->cht, &color, cmapP->permi[imin]);
+
     return cmapP->permi[imin];
 }
 
 
 
-static __inline__ int
-GetPixel(pixel ** const pixels, pixval const input_maxval,
-         gray ** const alpha, gray const alpha_threshold, 
-         struct cmap * const cmapP,
-         int const x, int const y) {
+enum pass {MULT8PLUS0, MULT8PLUS4, MULT4PLUS2, MULT2PLUS1};
+
+
+struct pixelCursor {
+    unsigned int width;
+        /* Width of the image, in columns */
+    unsigned int height;
+        /* Height of the image, in rows */
+    bool interlace;
+        /* We're accessing the image in interlace fashion */
+    unsigned int nPixelsLeft;
+        /* Number of pixels left to be read */
+    unsigned int curCol;
+        /* Location of pointed-to pixel, column */
+    unsigned int curRow;
+        /* Location of pointed-to pixel, row */
+    enum pass pass;
+        /* The interlace pass.  Undefined if !interlace */
+};
+
+
+
+static struct pixelCursor pixelCursor;
+    /* Current location in the input pixels.  */
+
+
+static void
+initPixelCursor(unsigned int const width,
+                unsigned int const height,
+                bool         const interlace) {
+
+    pixelCursor.width       = width;
+    pixelCursor.height      = height;
+    pixelCursor.interlace   = interlace;
+    pixelCursor.pass        = MULT8PLUS0;
+    pixelCursor.curCol      = 0;
+    pixelCursor.curRow      = 0;
+    pixelCursor.nPixelsLeft = width * height;
+}
+
+
+
+static void
+getPixel(pixel **           const pixels,
+         pixval             const inputMaxval,
+         gray **            const alpha,
+         gray               const alphaThreshold, 
+         struct cmap *      const cmapP,
+         struct pixelCursor const pixelCursor,
+         int *              const retvalP) {
 /*----------------------------------------------------------------------------
-   Return the colormap index of the pixel at location (x,y) in the PPM
-   raster 'pixels', using colormap *cmapP.
+   Return as *retvalP the colormap index of the pixel at location
+   pointed to by 'pixelCursor' in the PPM raster 'pixels', using
+   colormap *cmapP.
 -----------------------------------------------------------------------------*/
+    unsigned int const x = pixelCursor.curCol;
+    unsigned int const y = pixelCursor.curRow;
+
     int colorindex;
 
-    if (alpha && alpha[y][x] < alpha_threshold)
+    if (alpha && alpha[y][x] < alphaThreshold)
         colorindex = cmapP->transparent;
     else {
-        int presort_colorindex;
+        int presortColorindex;
 
-        presort_colorindex = ppm_lookupcolor(cmapP->cht, &pixels[y][x]);
-        if (presort_colorindex == -1)
-            presort_colorindex = 
-                closestcolor(pixels[y][x], input_maxval, cmapP);
-        colorindex = cmapP->perm[presort_colorindex];
+        presortColorindex = ppm_lookupcolor(cmapP->cht, &pixels[y][x]);
+        if (presortColorindex == -1)
+            presortColorindex = 
+                closestcolor(pixels[y][x], inputMaxval, cmapP);
+        colorindex = cmapP->perm[presortColorindex];
     }
-    return colorindex;
+    *retvalP = colorindex;
 }
 
 
-#define TRUE 1
-#define FALSE 0
 
-static int Width, Height;
-static int curx, cury;
-static long CountDown;
-static int Pass = 0;
-static int Interlace;
-
-/*
- * Bump the 'curx' and 'cury' to point to the next pixel
- */
 static void
-BumpPixel()
-{
-        /*
-         * Bump the current X position
-         */
-        ++curx;
-
-        /*
-         * If we are at the end of a scan line, set curx back to the beginning
-         * If we are interlaced, bump the cury to the appropriate spot,
-         * otherwise, just increment it.
-         */
-        if( curx == Width ) {
-                curx = 0;
-
-                if( !Interlace )
-                        ++cury;
-                else {
-                     switch( Pass ) {
-
-                       case 0:
-                          cury += 8;
-                          if( cury >= Height ) {
-                                ++Pass;
-                                cury = 4;
-                          }
-                          break;
-
-                       case 1:
-                          cury += 8;
-                          if( cury >= Height ) {
-                                ++Pass;
-                                cury = 2;
-                          }
-                          break;
-
-                       case 2:
-                          cury += 4;
-                          if( cury >= Height ) {
-                             ++Pass;
-                             cury = 1;
-                          }
-                          break;
-
-                       case 3:
-                          cury += 2;
-                          break;
-                        }
-                }
+bumpRowInterlace(struct pixelCursor * const pixelCursorP) {
+/*----------------------------------------------------------------------------
+   Move *pixelCursorP to the next row in the interlace pattern.
+-----------------------------------------------------------------------------*/
+    /* There are 4 passes:
+       MULT8PLUS0: Rows 8, 16, 24, 32, etc.
+       MULT8PLUS4: Rows 4, 12, 20, 28, etc.
+       MULT4PLUS2: Rows 2, 6, 10, 14, etc.
+       MULT2PLUS1: Rows 1, 3, 5, 7, 9, etc.
+    */
+    
+    switch (pixelCursorP->pass) {
+    case MULT8PLUS0:
+        pixelCursorP->curRow += 8;
+        break;
+    case MULT8PLUS4:
+        pixelCursorP->curRow += 8;
+        break;
+    case MULT4PLUS2:
+        pixelCursorP->curRow += 4;
+        break;
+    case MULT2PLUS1:
+        pixelCursorP->curRow += 2;
+        break;
+    }
+    /* Set the proper pass for the next read.  Note that if there are
+       more than 4 rows, the sequence of passes is sequential, but
+       when there are fewer than 4, we may skip e.g. from MULT8PLUS0
+       to MULT4PLUS2.
+    */
+    while (pixelCursorP->curRow >= pixelCursorP->height) {
+        switch (pixelCursorP->pass) {
+        case MULT8PLUS0:
+            pixelCursorP->pass = MULT8PLUS4;
+            pixelCursorP->curRow = 4;
+            break;
+        case MULT8PLUS4:
+            pixelCursorP->pass = MULT4PLUS2;
+            pixelCursorP->curRow = 2;
+            break;
+        case MULT4PLUS2:
+            pixelCursorP->pass = MULT2PLUS1;
+            pixelCursorP->curRow = 1;
+            break;
+        case MULT2PLUS1:
+            /* We've read the entire image; pass and current row are
+               now undefined.
+            */
+            pixelCursorP->curRow = 0;
+            break;
         }
+    }
 }
 
 
 
-static __inline__ int
-GIFNextPixel(pixel ** const pixels, pixval const input_maxval,
-             gray ** const alpha, gray const alpha_threshold, 
+static void
+bumpPixel(struct pixelCursor * const pixelCursorP) {
+/*----------------------------------------------------------------------------
+   Bump *pixelCursorP to point to the next pixel to go into the GIF
+
+   Must not call when there are no pixels left.
+-----------------------------------------------------------------------------*/
+    assert(pixelCursorP->nPixelsLeft > 0);
+
+    /* Move one column to the right */
+    ++pixelCursorP->curCol;
+    
+    if (pixelCursorP->curCol >= pixelCursorP->width) {
+        /* That pushed us past the end of a row. */
+        /* Reset to the left edge ... */
+        pixelCursorP->curCol = 0;
+        
+        /* ... of the next row */
+        if (!pixelCursorP->interlace)
+            /* Go to the following row */
+            ++pixelCursorP->curRow;
+        else
+            bumpRowInterlace(pixelCursorP);
+    }
+    --pixelCursorP->nPixelsLeft;
+}
+
+
+
+static int
+gifNextPixel(pixel **      const pixels,
+             pixval        const inputMaxval,
+             gray **       const alpha,
+             gray          const alphaThreshold, 
              struct cmap * const cmapP) {
 /*----------------------------------------------------------------------------
    Return the pre-sort color index (index into the unsorted GIF color map)
@@ -367,19 +447,17 @@ GIFNextPixel(pixel ** const pixels, pixval const input_maxval,
    map whose value is less that that represents a transparent pixel
    in the output.
 -----------------------------------------------------------------------------*/
-    int r;
+    int retval;
 
-    if( CountDown == 0 )
-        return EOF;
-    
-    --CountDown;
-    
-    r = GetPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP, 
-                 curx, cury);
-    
-    BumpPixel();
-    
-    return r;
+    if (pixelCursor.nPixelsLeft == 0 )
+        retval = EOF;
+    else {
+        getPixel(pixels, inputMaxval, alpha, alphaThreshold, cmapP, 
+                 pixelCursor, &retval);
+
+        bumpPixel(&pixelCursor);
+    }
+    return retval;
 }
 
 
@@ -529,7 +607,6 @@ static long int out_count = 0;           /* # of codes output (for debugging) */
  */
 
 static int g_init_bits;
-static FILE* g_outfile;
 
 static int ClearCode;
 static int EOFCode;
@@ -538,42 +615,63 @@ static int EOFCode;
 *                          BYTE OUTPUTTER                 
 ***************************************************************************/
 
-static char accum[256];
-    /* The current packet, under construction */
-static int a_count;
-    /* Number of characters so far in the current packet */
+struct outputter {
+    FILE * fileP;  /* The file to which to output */
+    unsigned int count;
+        /* Number of bytes so far in the current data block */
+    unsigned char buffer[256];
+        /* The current data block, under construction */
+};
+
+static struct outputter outputter;
+
+
 
 static void
-char_init() {
-    a_count = 0;
+outputter_init(FILE * const fileP) {
+    outputter.fileP = fileP;
+    outputter.count = 0;
 }
 
 
 
 static void
-flush_char() {
+outputter_flush() {
 /*----------------------------------------------------------------------------
-   Write the current packet to the output file, then reset the current 
-   packet to empty.
+   Write the current data block to the output file, then reset the current 
+   data block to empty.
 -----------------------------------------------------------------------------*/
-    if( a_count > 0 ) {
-        fputc( a_count, g_outfile );
-        fwrite( accum, 1, a_count, g_outfile );
-        a_count = 0;
+    if (outputter.count > 0 ) {
+        if (verbose)
+            pm_message("Writing %u byte block", outputter.count);
+        fputc(outputter.count, outputter.fileP);
+        fwrite(outputter.buffer, 1, outputter.count, outputter.fileP);
+        outputter.count = 0;
     }
 }
 
 
 
 static void
-char_out( int const c ) {
+outputter_flushFile() {
+    
+    fflush(outputter.fileP);
+    
+    if (ferror(outputter.fileP))
+        pm_error("error writing output file");
+}
+
+
+
+static void
+outputter_out(unsigned char const c) {
 /*----------------------------------------------------------------------------
-   Add a character to the end of the current packet, and if it is 254
-   character, flush the packet to the output file.
+  Add a byte to the end of the current data block, and if it is now 254
+  characters, flush the data block to the output file.
 -----------------------------------------------------------------------------*/
-    accum[ a_count++ ] = c;
-    if( a_count >= 254 )
-        flush_char();
+    outputter.buffer[outputter.count++] = c;
+    if (outputter.count >= 254)
+        outputter_flush();
 }
 
 
@@ -584,87 +682,78 @@ static unsigned long const masks[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F,
                                        0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };
 
 static void
-output( code_int const code, FILE * const outfile ) {
+output(code_int const code) {
 /*----------------------------------------------------------------------------
    Output one GIF code to the file.
 
    The code is represented as cur_bits+n_bits/8 bytes in the file.
 
- * Inputs:
- *      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
- *              that n_bits =< (long)wordsize - 1.
- * Assumptions:
- *      Chars are 8 bits long.
- * Algorithm:
- *      Maintain a BITS character long buffer (so that 8 codes will
- * fit in it exactly).  Use the VAX insv instruction to insert each
- * code in turn.  When the buffer fills up empty it and start over.
-
+   Inputs:
+      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
+              that n_bits =< (long)wordsize - 1.
+   Assumptions:
+      Chars are 8 bits long.
 -----------------------------------------------------------------------------*/
+    /*
+      Algorithm:
+        Maintain a BITS character long buffer (so that 8 codes will
+        fit in it exactly).  Use the VAX insv instruction to insert each
+        code in turn.  When the buffer fills up empty it and start over.
+    */
+    
     /* Note: this initialization of static variables by the loader must go
        if we want to use the compressor more than once per invocation
        (e.g. we want to create a multi-image gif).
     */
-    static unsigned long cur_accum = 0;
-    static int cur_bits = 0;
+    static unsigned long curAccum;
+    static int curBits;
 
-    g_outfile = outfile;
+    curBits = 0;  /* initial value */
+    curAccum = 0;  /* initial value */
+    curAccum &= masks[curBits];
 
-    cur_accum &= masks[ cur_bits ];
-
-    if( cur_bits > 0 )
-        cur_accum |= ((long)code << cur_bits);
+    if (curBits > 0)
+        curAccum |= ((long)code << curBits);
     else
-        cur_accum = code;
+        curAccum = code;
 
-    cur_bits += n_bits;
+    curBits += n_bits;
 
-    while( cur_bits >= 8 ) {
-        char_out( (unsigned int)(cur_accum & 0xff) );
-        cur_accum >>= 8;
-        cur_bits -= 8;
+    while (curBits >= 8) {
+        outputter_out((unsigned int)(curAccum & 0xff));
+        curAccum >>= 8;
+        curBits -= 8;
     }
 
-    /*
-     * If the next entry is going to be too big for the code size,
-     * then increase it, if possible.
-     */
-   if ( free_ent > maxcode || clear_flg ) {
-
-            if( clear_flg ) {
-
-                maxcode = MAXCODE (n_bits = g_init_bits);
-                clear_flg = 0;
-
-            } else {
-
-                ++n_bits;
-                if ( n_bits == maxbits )
-                    maxcode = maxmaxcode;
-                else
-                    maxcode = MAXCODE(n_bits);
-            }
-        }
-
-    if( code == EOFCode ) {
-        /*
-         * At EOF, write the rest of the buffer.
-         */
-        while( cur_bits > 0 ) {
-                char_out( (unsigned int)(cur_accum & 0xff) );
-                cur_accum >>= 8;
-                cur_bits -= 8;  
-                    /* ?? what if cur_bits < 8?  Apparently only an issue 
-                       if we extend this program to do multiple images */
-        }
-
-        flush_char();
-
-        fflush( g_outfile );
-
-        if( ferror( g_outfile ) )
-            pm_error( "error writing output file" );
-    }
+    /* If the next entry is going to be too big for the code size,
+       then increase it, if possible.
+    */
+   if (free_ent > maxcode || clear_flg) {
+       if (clear_flg) {
+           maxcode = MAXCODE (n_bits = g_init_bits);
+           clear_flg = 0;
+       } else {
+           ++n_bits;
+           if (n_bits == maxbits)
+               maxcode = maxmaxcode;
+           else
+               maxcode = MAXCODE(n_bits);
+       }
+   }
+   
+   if (code == EOFCode) {
+       /* We're at EOF.  Write the rest of the buffer. */
+       while (curBits > 0) {
+           outputter_out((unsigned int)(curAccum & 0xff));
+           curAccum >>= 8;
+           curBits -= 8;  
+           /* ?? what if cur_bits < 8?  Apparently only an issue 
+              if we extend this program to do multiple images */
+       }
+       outputter_flush();
+       
+       outputter_flushFile();
+   }
 }
 
 
@@ -709,14 +798,14 @@ register count_int hsize;
  * Clear out the hash table
  */
 static void
-cl_block (FILE * const outfile)     /* table clear for block compress */
-{
+cl_block() {
+    /* table clear for block compress */
 
-        cl_hash ( (count_int) hsize );
-        free_ent = ClearCode + 2;
-        clear_flg = 1;
-
-        output( (code_int)ClearCode, outfile );
+    cl_hash ( (count_int) hsize );
+    free_ent = ClearCode + 2;
+    clear_flg = 1;
+    
+    output((code_int)ClearCode);
 }
 
 
@@ -750,7 +839,6 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
     
     /*
      * Set up the globals:  g_init_bits - initial number of bits
-     *                      g_outfile   - pointer to output file
      */
     g_init_bits = init_bits;
 
@@ -767,9 +855,9 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
     EOFCode = ClearCode + 1;
     free_ent = ClearCode + 2;
 
-    char_init();
+    outputter_init(outfile);
 
-    ent = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
+    ent = gifNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
 
     {
         long fcode;
@@ -781,7 +869,7 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
     hsize_reg = hsize;
     cl_hash( (count_int) hsize_reg);            /* clear hash table */
 
-    output( (code_int)ClearCode, outfile );
+    output((code_int)ClearCode);
 
     eof = FALSE;
     while (!eof) {
@@ -789,7 +877,7 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
             /* The value for the pixel in the GIF image.  I.e. the colormap
                index.  Or -1 to indicate "no more pixels."
             */
-        gifpixel = GIFNextPixel(pixels, 
+        gifpixel = gifNextPixel(pixels, 
                                 input_maxval, alpha, alpha_threshold, cmapP);
         if (gifpixel == EOF) eof = TRUE;
         if (!eof) {
@@ -820,7 +908,7 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
             if ( (long)HashTabOf (i) > 0 )
                 goto probe;
         nomatch:
-            output ( (code_int) ent, outfile );
+            output((code_int)ent);
             ++out_count;
             ent = gifpixel;
             if ( free_ent < maxmaxcode ) {  /* } */
@@ -833,9 +921,9 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
     /*
      * Put out the final code.
      */
-    output( (code_int)ent, outfile );
+    output((code_int)ent);
     ++out_count;
-    output( (code_int) EOFCode, outfile );
+    output((code_int) EOFCode);
 }
 
 
@@ -843,104 +931,109 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
 /* Routine to convert variable-width codes into a byte stream */
 
 static void
-output_uncompressed(struct gif_dest * const dinfo, int const code,
-                    FILE * const outfile) {
-
-    g_outfile = outfile;
+outputUncompressed(struct gif_dest * const dinfoP,
+                   int               const code) {
 
     /* Emit a code of n_bits bits */
     /* Uses cur_accum and cur_bits to reblock into 8-bit bytes */
-    dinfo->cur_accum |= ((int) code) << dinfo->cur_bits;
-    dinfo->cur_bits += dinfo->n_bits;
+    dinfoP->cur_accum |= ((int) code) << dinfoP->cur_bits;
+    dinfoP->cur_bits += dinfoP->n_bits;
 
-    while (dinfo->cur_bits >= 8) {
-        char_out(dinfo->cur_accum & 0xFF);
-        dinfo->cur_accum >>= 8;
-        dinfo->cur_bits -= 8;
+    while (dinfoP->cur_bits >= 8) {
+        outputter_out(dinfoP->cur_accum & 0xFF);
+        dinfoP->cur_accum >>= 8;
+        dinfoP->cur_bits -= 8;
     }
 }
 
 
 static void
-write_raster_uncompressed_init (struct gif_dest * const dinfo, 
-                                int const i_bits, FILE * const outfile)
-/* Initialize pseudo-compressor */
-{
-  /* init all the state variables */
-  dinfo->n_bits = i_bits;
-  dinfo->maxcode = MAXCODE(dinfo->n_bits);
-  dinfo->ClearCode = (1 << (i_bits - 1));
-  dinfo->EOFCode = dinfo->ClearCode + 1;
-  dinfo->code_counter = dinfo->ClearCode + 2;
-  /* init output buffering vars */
-  dinfo->cur_accum = 0;
-  dinfo->cur_bits = 0;
-  /* GIF specifies an initial Clear code */
-  output_uncompressed(dinfo, dinfo->ClearCode, outfile);
-}
-
-
-static void
-write_raster_uncompressed_pixel (struct gif_dest * const dinfo, 
-                                 int const c, FILE * const outfile)
-/* Accept and "compress" one pixel value.
- * The given value must be less than n_bits wide.
- */
-{
-  /* Output the given pixel value as a symbol. */
-  output_uncompressed(dinfo, c, outfile);
-  /* Issue Clear codes often enough to keep the reader from ratcheting up
-   * its symbol size.
-   */
-  if (dinfo->code_counter < dinfo->maxcode) {
-    dinfo->code_counter++;
-  } else {
-    output_uncompressed(dinfo, dinfo->ClearCode, outfile);
-    dinfo->code_counter = dinfo->ClearCode + 2;	/* reset the counter */
-  }
-}
-
-
-static void
-write_raster_uncompressed_term (struct gif_dest * const dinfo,
-                                FILE * const outfile)
-/* Clean up at end */
-{
-  /* Send an EOF code */
-  output_uncompressed(dinfo, dinfo->EOFCode, outfile);
-  /* Flush the bit-packing buffer */
-  if (dinfo->cur_bits > 0) {
-    char_out(dinfo->cur_accum & 0xFF);
-  }
-  /* Flush the packet buffer */
-  flush_char();
-}
-
-
-
-static void
-write_raster_uncompressed(pixel ** const pixels, pixval const input_maxval,
-                          gray ** const alpha, gray const alpha_maxval, 
-                          struct cmap * const cmapP, 
-                          int const init_bits, FILE * const outfile) {
+writeRasterUncompressedInit(FILE *            const ofP,
+                            struct gif_dest * const dinfoP, 
+                            int               const i_bits) {
 /*----------------------------------------------------------------------------
-   Write the raster to file 'outfile'.
+   Initialize pseudo-compressor
+-----------------------------------------------------------------------------*/
+
+    outputter_init(ofP);
+
+    /* init all the state variables */
+    dinfoP->n_bits = i_bits;
+    dinfoP->maxcode = MAXCODE(dinfoP->n_bits);
+    dinfoP->ClearCode = (1 << (i_bits - 1));
+    dinfoP->EOFCode = dinfoP->ClearCode + 1;
+    dinfoP->code_counter = dinfoP->ClearCode + 2;
+    /* init output buffering vars */
+    dinfoP->cur_accum = 0;
+    dinfoP->cur_bits = 0;
+    /* GIF specifies an initial Clear code */
+    outputUncompressed(dinfoP, dinfoP->ClearCode);
+}
+
+
+
+static void
+writeRasterUncompressedPixel(struct gif_dest * const dinfoP, 
+                             unsigned int      const colormapIndex) {
+/*----------------------------------------------------------------------------
+   "Compress" one pixel value and output it as a symbol.
+
+   'colormapIndex' must be less than dinfoP->n_bits wide.
+-----------------------------------------------------------------------------*/
+    assert(colormapIndex >> dinfoP->n_bits == 0);
+
+    outputUncompressed(dinfoP, colormapIndex);
+    /* Issue Clear codes often enough to keep the reader from ratcheting up
+     * its symbol size.
+     */
+    if (dinfoP->code_counter < dinfoP->maxcode) {
+        ++dinfoP->code_counter;
+    } else {
+        outputUncompressed(dinfoP, dinfoP->ClearCode);
+        dinfoP->code_counter = dinfoP->ClearCode + 2;	/* reset the counter */
+    }
+}
+
+
+
+static void
+writeRasterUncompressedTerm(struct gif_dest * const dinfoP) {
+
+    outputUncompressed(dinfoP, dinfoP->EOFCode);
+
+    if (dinfoP->cur_bits > 0)
+        outputter_out(dinfoP->cur_accum & 0xFF);
+
+    outputter_flush();
+}
+
+
+
+static void
+writeRasterUncompressed(FILE *         const ofP, 
+                        pixel **       const pixels,
+                        pixval         const inputMaxval,
+                        gray **        const alpha,
+                        gray           const alphaMaxval, 
+                        struct cmap *  const cmapP, 
+                        int            const initBits) {
+/*----------------------------------------------------------------------------
+   Write the raster to file 'ofP'.
    
    Same as write_raster_LZW(), except written out one code per
    pixel (plus some clear codes), so no compression.  And no use
    of the LZW patent.
 -----------------------------------------------------------------------------*/
-    gray const alpha_threshold = (alpha_maxval + 1) / 2;
+    gray const alphaThreshold = (alphaMaxval + 1) / 2;
         /* gray levels below this in the alpha mask indicate transparent
            pixels in the output image.
         */
     bool eof;
 
-    struct gif_dest gif_dest;
+    struct gif_dest gifDest;
 
-    write_raster_uncompressed_init(&gif_dest, init_bits, outfile);
-
-    g_outfile = outfile;
+    writeRasterUncompressedInit(ofP, &gifDest, initBits);
+    pm_message("before raster, position is %lu", ftell(outputter.fileP));
 
     eof = FALSE;
     while (!eof) {
@@ -948,14 +1041,15 @@ write_raster_uncompressed(pixel ** const pixels, pixval const input_maxval,
             /* The value for the pixel in the GIF image.  I.e. the colormap
                index.  Or -1 to indicate "no more pixels."
             */
-        gifpixel = GIFNextPixel(pixels, 
-                                input_maxval, alpha, alpha_threshold, cmapP);
-        if (gifpixel == EOF) eof = TRUE;
-        if (!eof) {
-            write_raster_uncompressed_pixel(&gif_dest, gifpixel, outfile);
-        }
+        gifpixel = gifNextPixel(pixels, 
+                                inputMaxval, alpha, alphaThreshold, cmapP);
+        if (gifpixel == EOF)
+            eof = TRUE;
+        else
+            writeRasterUncompressedPixel(&gifDest, gifpixel);
     }
-    write_raster_uncompressed_term(&gif_dest, outfile);
+    writeRasterUncompressedTerm(&gifDest);
+    pm_message("after writing raster, position is %lu", ftell(outputter.fileP));
 }
 
 
@@ -977,15 +1071,6 @@ writeGifHeader(FILE * const fp,
     int const Resolution = BitsPerPixel;
     int const ColorMapSize = 1 << BitsPerPixel;
 
-    /* Calculate number of bits we are expecting */
-    CountDown = (long)Width * (long)Height;
-
-    /* Indicate which pass we are on (if interlace) */
-    Pass = 0;
-
-    /* Set up the current x and y position */
-    curx = cury = 0;
-
     /* Write the Magic header */
     if (cmapP->transparent != -1 || comment)
         fwrite("GIF89a", 1, 6, fp);
@@ -1044,69 +1129,78 @@ writeGifHeader(FILE * const fp,
 
 
 static void
-GIFEncode(FILE * const fp, 
-          pixel ** const pixels, pixval const input_maxval,
-          int const GWidth, int const GHeight, 
-          gray ** const alpha, gray const alpha_maxval,
-          int const GInterlace, int const Background, 
-          int const BitsPerPixel, struct cmap * const cmapP,
-          const char comment[], bool const nolzw) {
+writeImageHeader(FILE *       const ofP,
+                 unsigned int const leftOffset,
+                 unsigned int const topOffset,
+                 unsigned int const gWidth,
+                 unsigned int const gHeight,
+                 unsigned int const gInterlace,
+                 unsigned int const initCodeSize) {
 
-    int const LeftOfs = 0;
-    int const TopOfs = 0;
-    int InitCodeSize;
-    
-    writeGifHeader(fp, GWidth, GHeight, GInterlace, Background,
-                   BitsPerPixel, cmapP, comment);
-
-    /* Write an Image separator */
-    fputc( ',', fp );
-
-    /* Write the Image header */
-
-    Putword( LeftOfs, fp );
-    Putword( TopOfs, fp );
-    Putword( GWidth, fp );
-    Putword( GHeight, fp );
+    Putword(leftOffset, ofP);
+    Putword(topOffset,  ofP);
+    Putword(gWidth,     ofP);
+    Putword(gHeight,    ofP);
 
     /* Write out whether or not the image is interlaced */
-    if( GInterlace )
-        fputc( 0x40, fp );
+    if (gInterlace)
+        fputc(0x40, ofP);
     else
-        fputc( 0x00, fp );
-
-    /* The initial code size */
-    if( BitsPerPixel <= 1 )
-        InitCodeSize = 2;
-    else
-        InitCodeSize = BitsPerPixel;
+        fputc(0x00, ofP);
 
     /* Write out the initial code size */
-    fputc( InitCodeSize, fp );
+    fputc(initCodeSize, ofP);
+}
 
-    /* Set some global variables for BumpPixel() */
-    Interlace = GInterlace;
-    Width = GWidth;
-    Height = GHeight;
+
+
+static void
+gifEncode(FILE *        const ofP, 
+          pixel **      const pixels,
+          pixval        const inputMaxval,
+          int           const gWidth,
+          int           const gHeight, 
+          gray **       const alpha,
+          gray          const alphaMaxval,
+          int           const gInterlace,
+          int           const background, 
+          int           const bitsPerPixel,
+          struct cmap * const cmapP,
+          char          const comment[],
+          bool          const nolzw) {
+
+    unsigned int const leftOffset = 0;
+    unsigned int const topOffset  = 0;
+
+    unsigned int const initCodeSize = bitsPerPixel <= 1 ? 2 : bitsPerPixel;
+        /* The initial code size */
+
+    writeGifHeader(ofP, gWidth, gHeight, gInterlace, background,
+                   bitsPerPixel, cmapP, comment);
+
+    /* Write an Image separator */
+    fputc(',', ofP);
+
+    writeImageHeader(ofP, leftOffset, topOffset, gWidth, gHeight, gInterlace,
+                     initCodeSize);
+
+    initPixelCursor(gWidth, gHeight, gInterlace);
 
     /* Write the actual raster */
     if (nolzw)
-        write_raster_uncompressed(pixels, 
-                                  input_maxval, alpha, alpha_maxval, cmapP, 
-                                  InitCodeSize+1, fp);
+        writeRasterUncompressed(ofP, pixels, 
+                                inputMaxval, alpha, alphaMaxval, cmapP, 
+                                initCodeSize + 1);
     else
         write_raster_LZW(pixels, 
-                         input_maxval, alpha, alpha_maxval, cmapP, 
-                         InitCodeSize+1, fp);
+                         inputMaxval, alpha, alphaMaxval, cmapP, 
+                         initCodeSize + 1, ofP);
 
-    /* Write out a Zero-length packet (to end the series) */
-    fputc( 0, fp );
+    /* Write out a zero length data block (to end the series) */
+    fputc(0, ofP);
 
     /* Write the GIF file terminator */
-    fputc( ';', fp );
-
-    /* And close the file */
-    fclose( fp );
+    fputc(';', ofP);
 }
 
 
@@ -1283,6 +1377,7 @@ colormap_from_file(const char filespec[], unsigned int const maxcolors,
 }
 
 
+
 static void
 get_alpha(const char * const alpha_filespec, int const cols, int const rows,
           gray *** const alphaP, gray * const maxvalP) {
@@ -1300,6 +1395,7 @@ get_alpha(const char * const alpha_filespec, int const cols, int const rows,
 }
 
 
+
 static void
 compute_ppm_colormap(pixel ** const pixels, int const cols, int const rows,
                      int const input_maxval, bool const have_alpha, 
@@ -1356,8 +1452,8 @@ compute_ppm_colormap(pixel ** const pixels, int const cols, int const rows,
 
 int
 main(int argc, char *argv[]) {
-    struct cmdline_info cmdline;
-    FILE* ifp;
+    struct cmdlineInfo cmdline;
+    FILE * ifP;
     int rows, cols;
     int BitsPerPixel;
     pixel ** pixels;   /* The input image, in PPM format */
@@ -1376,13 +1472,15 @@ main(int argc, char *argv[]) {
 
     ppm_init( &argc, argv );
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.input_filespec);
+    verbose = cmdline.verbose;
 
-    pixels = ppm_readppm(ifp, &cols, &rows, &input_maxval);
+    ifP = pm_openr(cmdline.input_filespec);
 
-    pm_close(ifp);
+    pixels = ppm_readppm(ifP, &cols, &rows, &input_maxval);
+
+    pm_close(ifP);
 
     get_alpha(cmdline.alpha_filespec, cols, rows, &alpha, &alpha_maxval);
 
@@ -1416,7 +1514,7 @@ main(int argc, char *argv[]) {
             cmap.transparent = -1;
     }
     /* All set, let's do it. */
-    GIFEncode(stdout, pixels, input_maxval, cols, rows, 
+    gifEncode(stdout, pixels, input_maxval, cols, rows, 
               alpha, alpha_maxval, 
               cmdline.interlace, 0, BitsPerPixel, &cmap, cmdline.comment,
               cmdline.nolzw);
@@ -1425,7 +1523,7 @@ main(int argc, char *argv[]) {
     if (alpha)
         pgm_freearray(alpha, rows);
 
-    exit(0);
+    fclose(stdout);
+
+    return 0;
 }
-
-
