@@ -104,9 +104,6 @@ parseCommandLine ( int argc, char ** argv,
     if (!outstemSpec)
         pm_error("You must specify the -outstem option to indicate where to "
                  "put the output images.");
-    else if (strlen(cmdlineP->outstem) > MAXFILENAMELEN - 10)
-        pm_error("-outstem value is too long.  That would exceed the maximum "
-                 "allowable filename length of %u", MAXFILENAMELEN);
     if (argc-1 < 1)
         cmdlineP->inputFilespec = "-";
     else if (argc-1 == 1)
@@ -174,13 +171,6 @@ computeSliceGeometry(struct cmdlineInfo const cmdline,
     *rightSliceWidthP = 
         inpam.width - (*nVertSliceP-1) * (cmdline.width - cmdline.hoverlap);
 
-    if (*nVertSliceP > 100)
-        pm_error("You have %d vertical slices.  The maximum allowed is 100",
-                 *nVertSliceP);
-    if (*nHorizSliceP > 100)
-        pm_error("You have %d horizontal slices.  The maximum allowed is 100",
-                 *nHorizSliceP);
-
     if (verbose) {
         pm_message("Creating %u images, %u across by %u down; "
                    "each %u w x %u h",
@@ -198,10 +188,54 @@ computeSliceGeometry(struct cmdlineInfo const cmdline,
 
 
 
+static unsigned int
+ndigits(unsigned int const arg) {
+/*----------------------------------------------------------------------------
+   Return the minimum number of digits it takes to represent the number
+   'arg' in decimal.
+-----------------------------------------------------------------------------*/
+    unsigned int leftover;
+    unsigned int i;
+
+    for (leftover = arg, i = 0; leftover > 0; leftover /= 10, ++i);
+
+    return MAX(1, i);
+}
+
+
+
+static void
+computeOutputFilenameFormat(int           const format, 
+                            char          const outstem[],
+                            unsigned int  const nHorizSlice,
+                            unsigned int  const nVertSlice,
+                            const char ** const filenameFormatP) {
+
+    const char * filenameSuffix;
+
+    switch(PNM_FORMAT_TYPE(format)) {
+    case PPM_TYPE: filenameSuffix = "ppm"; break;
+    case PGM_TYPE: filenameSuffix = "pgm"; break;
+    case PBM_TYPE: filenameSuffix = "pbm"; break;
+    case PAM_TYPE: filenameSuffix = "pam"; break;
+    default:       filenameSuffix = "";    break;
+    }
+    
+    asprintfN(filenameFormatP, "%s_%%0%uu_%%0%uu.%s",
+              outstem, ndigits(nHorizSlice), ndigits(nVertSlice),
+              filenameSuffix);
+
+    if (*filenameFormatP == NULL)
+        pm_error("Unable to allocate memory for filename format string");
+}
+
+
+
 static void
 openOutStreams(struct pam   const inpam, 
                struct pam         outpam[],
                unsigned int const horizSlice, 
+               unsigned int const nHorizSlice,
                unsigned int const nVertSlice,
                unsigned int const sliceHeight, 
                unsigned int const sliceWidth,
@@ -212,45 +246,35 @@ openOutStreams(struct pam   const inpam,
    Open the output files for a single horizontal slice (there's one file
    for each vertical slice) and write the Netpbm headers to them.  Also
    compute the pam structures to control each.
-
-   Assume 'horizSlice' is less than 100 and 'nVertSlice' is less than or
-   equal to 100.
-
-   Assume the output filename (controlled by 'outstem') fits in 
-   MAXFILENAMELEN characters.
 -----------------------------------------------------------------------------*/
+    const char * filenameFormat;
     unsigned int vertSlice;
 
+    computeOutputFilenameFormat(inpam.format, outstem, nHorizSlice, nVertSlice,
+                                &filenameFormat);
+
     for (vertSlice = 0; vertSlice < nVertSlice; ++vertSlice) {
-        char filename[MAXFILENAMELEN+1];
-        const char * filenameSuffix;
-        int rc;
+        const char * filename;
 
-        switch(PNM_FORMAT_TYPE(inpam.format)) {
-        case PPM_TYPE: filenameSuffix = "ppm"; break;
-        case PGM_TYPE: filenameSuffix = "pgm"; break;
-        case PBM_TYPE: filenameSuffix = "pbm"; break;
-        case PAM_TYPE: filenameSuffix = "pam"; break;
-        default:       filenameSuffix = "";    break;
+        asprintfN(&filename, filenameFormat, horizSlice, vertSlice);
+
+        if (filename == NULL)
+            pm_error("Unable to allocate memory for output filename");
+        else {
+            outpam[vertSlice] = inpam;
+            outpam[vertSlice].file = pm_openw(filename);
+            
+            outpam[vertSlice].width = 
+                vertSlice < nVertSlice-1 ? sliceWidth : rightSliceWidth;
+            
+            outpam[vertSlice].height = sliceHeight;
+            
+            pnm_writepaminit(&outpam[vertSlice]);
+
+            strfree(filename);
         }
-        
-        if (vertSlice > 99 || horizSlice > 99)
-            pm_error("Internal error: slice number doesn't fit in filename");
-        rc = snprintfN(filename, sizeof(filename), "%s_%02d_%02d.%s",
-                       outstem, horizSlice, vertSlice, filenameSuffix);
-        if (rc > sizeof(filename))
-            pm_error("Internal error: generated filename is too long.");
-
-        outpam[vertSlice] = inpam;
-        outpam[vertSlice].file = pm_openw(filename);
-
-        outpam[vertSlice].width = 
-            vertSlice < nVertSlice-1 ? sliceWidth : rightSliceWidth;
-
-        outpam[vertSlice].height = sliceHeight;
-
-        pnm_writepaminit(&outpam[vertSlice]);
     }        
+    strfree(filenameFormat);
 }
 
 
@@ -371,6 +395,23 @@ getInputRow(struct inputWindow * const inputWindowP,
 
 
 
+static void
+allocOutpam(unsigned int  const nVertSlice,
+            struct pam ** const outpamArrayP) {
+
+    struct pam * outpamArray;
+
+    MALLOCARRAY(outpamArray, nVertSlice);
+
+    if (outpamArray == NULL)
+        pm_error("Unable to allocate array for %u output pam structures.",
+                 nVertSlice);
+
+    *outpamArrayP = outpamArray;
+}
+
+
+
 int
 main(int argc, char ** argv) {
 
@@ -401,9 +442,9 @@ main(int argc, char ** argv) {
     unsigned int nVertSlice;
     struct inputWindow inputWindow;
     
-    struct pam outpam[100];
-        /* outpam[x] is the pam structure that controls the current horizontal
-           slice of vertical slice x.
+    struct pam * outpam;
+        /* malloc'ed.  outpam[x] is the pam structure that controls
+           the current horizontal slice of vertical slice x.
         */
 
     pnm_init(&argc, argv);
@@ -417,6 +458,8 @@ main(int argc, char ** argv) {
     computeSliceGeometry(cmdline, inpam, !!cmdline.verbose,
                          &nHorizSlice, &sliceHeight, &bottomSliceHeight,
                          &nVertSlice, &sliceWidth, &rightSliceWidth);
+
+    allocOutpam(nVertSlice, &outpam);
     
     initInput(&inputWindow, &inpam, 
               nHorizSlice > 1 ? cmdline.voverlap + 1 : 1);
@@ -429,7 +472,7 @@ main(int argc, char ** argv) {
 
         unsigned int row;
 
-        openOutStreams(inpam, outpam, horizSlice, nVertSlice, 
+        openOutStreams(inpam, outpam, horizSlice, nHorizSlice, nVertSlice, 
                        thisSliceHeight, sliceWidth, rightSliceWidth,
                        cmdline.hoverlap, cmdline.outstem);
 
@@ -443,11 +486,9 @@ main(int argc, char ** argv) {
 
     termInputWindow(&inputWindow);
 
+    free(outpam);
+
     pm_close(ifP);
 
     return 0;
 }
-
-
-
-
