@@ -21,7 +21,7 @@
      transformRowsBottomTopNonPbm()
      transformPbm()
    
-   Although transformGen() is used only when none of the enhancement
+   Although we use transformGen() only when none of the enhancement
    functions apply, it is capable of handling all cases.  (Only that it
    is slow, and uses more memory.)  In the same manner, transformPbm() is
    capable of handling all pbm transformations and transformRowByRowNonPbm()
@@ -354,42 +354,42 @@ bitOrderReverse(unsigned char * const bitrow,
   Reverse the bits in a packed pbm row (1 bit per pixel).  I.e. the leftmost
   bit becomes the rightmost, etc.
 -----------------------------------------------------------------------------*/
-    unsigned int const lastfullByteIdx = cols/8 -1;
+    unsigned int const lastfullByteIdx = cols/8 - 1;
 
     if (cols == 0 || bitrow == NULL )
         pm_error("Invalid arguments passed to bitOrderReverse");
 
     if (cols <= 8)
         bitrow[0] = bitreverse[bitrow[0]] << (8-cols);
-    else if (cols%8 == 0) {
+    else if (cols % 8 == 0) {
         unsigned int i, j;
         for (i = 0, j = lastfullByteIdx; i <= j; ++i, --j) {
             unsigned char const t = bitreverse[bitrow[j]]; 
-            bitrow[j]=bitreverse[bitrow[i]];
-            bitrow[i]=t;
+            bitrow[j] = bitreverse[bitrow[i]];
+            bitrow[i] = t;
         }
     } else {
-        unsigned int const m = cols%8; 
+        unsigned int const m = cols % 8; 
 
-        unsigned int i,j;
+        unsigned int i, j;
             /* Cursors into bitrow[].  i moves from left to center;
                j moves from right to center as bits of bitrow[] are exchanged.
             */
-        unsigned char th,tl;  /* 16 bit temp ( th << 8 | tl ) */
+        unsigned char th, tl;  /* 16 bit temp ( th << 8 | tl ) */
         tl = 0;
         for (i = 0, j = lastfullByteIdx+1; i <= lastfullByteIdx/2; ++i, --j) {
             th = bitreverse[bitrow[i]];
             bitrow[i] =
-                bitreverse[0xff & ((bitrow[j-1]<<8 | bitrow[j]) >> (8-m))];
-            bitrow[j] = 0xff & ((th<<8 | tl) >> m);
-            tl=th;
+                bitreverse[0xff & ((bitrow[j-1] << 8 | bitrow[j]) >> (8-m))];
+            bitrow[j] = 0xff & ((th << 8 | tl) >> m);
+            tl = th;
         }
         if (i == j) 
             /* bitrow[] has an odd number of bytes (an even number of
                full bytes; lastfullByteIdx is odd), so we did all but
                the center byte above.  We do the center byte now.
             */
-            bitrow[j] = 0xff & ((bitreverse[bitrow[i]]<<8|tl ) >> m );
+            bitrow[j] = 0xff & ((bitreverse[bitrow[i]] << 8 | tl) >> m);
     }
 }
 
@@ -889,7 +889,7 @@ main(int argc, char * argv[]) {
         */
         transformRowByRow(&inpam, &outpam, xform.a == -1, cmdline.verbose);
     else if (xform.b == 0 && xform.c == 0) 
-        /* In this case, Row N of the output is based only on Row ~M of the
+        /* In this case, Row N of the output is based only on Row ~N of the
            input.  We need all the rows in memory, but have to pass
            through them only twice, so there is no page thrashing concern.
         */
