@@ -388,7 +388,7 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
      * in a form easy for the program to use.
      */
-    const char *input_filespec;  /* Filespec of input file */
+    const char *inputFileName;  /* Filespec of input file */
     unsigned int nomix;
     basicFunction_t filterFunction; /* NULL if not using resample method */
     basicFunction_t windowFunction;
@@ -454,33 +454,33 @@ processFilterOptions(unsigned int const         filterSpec,
                      const char                 filterOpt[],
                      unsigned int const         windowSpec,
                      const char                 windowOpt[],
-                     struct cmdlineInfo * const cmdline_p) {
+                     struct cmdlineInfo * const cmdlineP) {
 
     if (filterSpec) {
         filter baseFilter;
         lookupFilterByName(filterOpt, &baseFilter);
-        cmdline_p->filterFunction = baseFilter.function; 
-        cmdline_p->filterRadius   = baseFilter.radius;
+        cmdlineP->filterFunction = baseFilter.function; 
+        cmdlineP->filterRadius   = baseFilter.radius;
 
         if (windowSpec) {
             filter windowFilter;
             lookupFilterByName(windowOpt, &windowFilter);
             
-            if (cmdline_p->windowFunction == filter_box)
-                cmdline_p->windowFunction = NULL;
+            if (cmdlineP->windowFunction == filter_box)
+                cmdlineP->windowFunction = NULL;
             else
-                cmdline_p->windowFunction = windowFilter.function;
+                cmdlineP->windowFunction = windowFilter.function;
         } else {
             /* Default for most filters is no window.  Those that _require_
                a window function get Blackman.
                */
             if (baseFilter.windowed)
-                cmdline_p->windowFunction = filter_blackman;
+                cmdlineP->windowFunction = filter_blackman;
             else
-                cmdline_p->windowFunction = NULL;
+                cmdlineP->windowFunction = NULL;
         }
     } else
-        cmdline_p->filterFunction = NULL;
+        cmdlineP->filterFunction = NULL;
 }
 
 
@@ -488,7 +488,7 @@ processFilterOptions(unsigned int const         filterSpec,
 static void
 parseXyParms(int                  const argc, 
              char **              const argv,
-             struct cmdlineInfo * const cmdline_p) {
+             struct cmdlineInfo * const cmdlineP) {
 
     /* parameters are box width (columns), box height (rows), and
        optional filespec 
@@ -502,26 +502,26 @@ parseXyParms(int                  const argc,
                  "you need 2 or 3 arguments.");
     else {
         char * endptr;
-        cmdline_p->xsize = strtol(argv[1], &endptr, 10);
+        cmdlineP->xsize = strtol(argv[1], &endptr, 10);
         if (strlen(argv[1]) > 0 && *endptr != '\0')
             pm_error("horizontal size argument not an integer: '%s'", 
                      argv[1]);
-        if (cmdline_p->xsize <= 0)
+        if (cmdlineP->xsize <= 0)
             pm_error("horizontal size argument is not positive: %d", 
-                     cmdline_p->xsize);
+                     cmdlineP->xsize);
         
-        cmdline_p->ysize = strtol(argv[2], &endptr, 10);
+        cmdlineP->ysize = strtol(argv[2], &endptr, 10);
         if (strlen(argv[2]) > 0 && *endptr != '\0')
             pm_error("vertical size argument not an integer: '%s'", 
                      argv[2]);
-        if (cmdline_p->ysize <= 0)
+        if (cmdlineP->ysize <= 0)
             pm_error("vertical size argument is not positive: %d", 
-                     cmdline_p->ysize);
+                     cmdlineP->ysize);
         
         if (argc-1 < 3)
-            cmdline_p->input_filespec = "-";
+            cmdlineP->inputFileName = "-";
         else
-            cmdline_p->input_filespec = argv[3];
+            cmdlineP->inputFileName = argv[3];
     }
 }
 
@@ -530,23 +530,23 @@ parseXyParms(int                  const argc,
 static void
 parseScaleParms(int                   const argc, 
                 char **               const argv,
-                struct cmdlineInfo  * const cmdline_p) {
+                struct cmdlineInfo  * const cmdlineP) {
 
     /* parameters are scale factor and optional filespec */
     if (argc-1 < 1)
         pm_error("With no dimension options, you must supply at least "
                  "one parameter: the scale factor.");
     else {
-        cmdline_p->xscale = cmdline_p->yscale = atof(argv[1]);
+        cmdlineP->xscale = cmdlineP->yscale = atof(argv[1]);
         
-        if (cmdline_p->xscale == 0.0)
+        if (cmdlineP->xscale == 0.0)
             pm_error("The scale parameter %s is not a positive number.",
                      argv[1]);
         else {
             if (argc-1 < 2)
-                cmdline_p->input_filespec = "-";
+                cmdlineP->inputFileName = "-";
             else
-                cmdline_p->input_filespec = argv[2];
+                cmdlineP->inputFileName = argv[2];
         }
     }
 }
@@ -556,13 +556,13 @@ parseScaleParms(int                   const argc,
 static void
 parseFilespecOnlyParms(int                   const argc, 
                        char **               const argv,
-                       struct cmdlineInfo  * const cmdline_p) {
+                       struct cmdlineInfo  * const cmdlineP) {
 
     /* Only parameter allowed is optional filespec */
     if (argc-1 < 1)
-        cmdline_p->input_filespec = "-";
+        cmdlineP->inputFileName = "-";
     else
-        cmdline_p->input_filespec = argv[1];
+        cmdlineP->inputFileName = argv[1];
 }
 
 
@@ -2085,7 +2085,7 @@ main(int argc, char **argv ) {
 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifP = pm_openr(cmdline.input_filespec);
+    ifP = pm_openr(cmdline.inputFileName);
 
     pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
 
