@@ -806,15 +806,32 @@ getScript(struct cmdlineInfo const cmdline,
 
           
 
+static void
+doOneImage(FILE *          const ifP,
+           struct script * const scriptP)  {
+
+    pixel ** pixels;
+    pixval maxval;
+    int rows, cols;
+    
+    pixels = ppm_readppm(ifP, &cols, &rows, &maxval);
+    
+    executeScript(scriptP, pixels, cols, rows, maxval);
+    
+    ppm_writeppm(stdout, pixels, cols, rows, maxval, 0);
+    
+    ppm_freearray(pixels, rows);
+}
+
+
+
 int
 main(int argc, char * argv[]) {
 
     struct cmdlineInfo cmdline;
     FILE * ifP;
-    pixel ** pixels;
-    pixval maxval;
-    int rows, cols;
     struct script * scriptP;
+    bool eof;
 
     ppm_init(&argc, argv);
 
@@ -824,18 +841,16 @@ main(int argc, char * argv[]) {
 
     ifP = pm_openr(cmdline.inputFilename);
 
-    pixels = ppm_readppm(ifP, &cols, &rows, &maxval);
-
     getScript(cmdline, &scriptP);
     
-    executeScript(scriptP, pixels, cols, rows, maxval);
+    eof = FALSE;
+    while (!eof) {
+        doOneImage(ifP, scriptP);
+        ppm_nextimage(ifP, &eof);
+    }
 
     freeScript(scriptP);
 
-    ppm_writeppm(stdout, pixels, cols, rows, maxval, 0);
-
-    ppm_freearray(pixels, rows);
-
     pm_close(ifP);
 
     /* If the program failed, it previously aborted with nonzero completion
