@@ -254,14 +254,14 @@ pnm_readpamrow(const struct pam * const pamP,
 
 
 tuple ** 
-pnm_readpam(FILE *       const file, 
+pnm_readpam(FILE *       const fileP,
             struct pam * const pamP, 
             int          const size) {
 
     tuple **tuplearray;
     int row;
 
-    pnm_readpaminit(file, pamP, size);
+    pnm_readpaminit(fileP, pamP, size);
     
     tuplearray = pnm_allocpamarray(pamP);
     
