@@ -45,17 +45,23 @@ struct pam {
            The information starts in the first byte and is contiguous.  
            This cannot be greater than 'size'
            */
-    FILE *file;
+    FILE * file;
     int format;
         /* The format code of the raw image.  This is PAM_FORMAT
            unless the PAM image is really a view of a PBM, PGM, or PPM
            image.  Then it's PBM_FORMAT, RPBM_FORMAT, etc.
            */
     unsigned int plainformat;
-        /* Logical:  the format above is a plain (text) format as opposed
-           to a raw (binary) format.  This is entirely redundant with the
-           'format' member and exists as a separate member only for 
-           computational speed.
+        /* Logical: On output, use the plain version of the format type
+           indicated by 'format'.  Otherwise, use the raw version.
+           (i.e., on output, the plainness information in 'format' is
+           irrelevant).  Input functions set this to FALSE, for the
+           convenience of programs that copy an input pam structure for
+           use with output.
+
+           Before Netpbm 10.32, this was rather different.  It simply
+           described for convenience the plainness of the format indicated
+           by 'format'.
         */
     int height;  /* Height of image in rows */
     int width;   
