@@ -10,8 +10,8 @@
 ** implied warranty.
 */
 
-#ifndef _PM_H_
-#define _PM_H_
+#ifndef PM_H_INCLUDED
+#define PM_H_INCLUDED
 
 #include "pm_config.h"
 
@@ -79,6 +79,14 @@ extern "C" {
 #define ARGS(alist) alist
 
 
+typedef struct {
+    /* Coordinates of a pixel within an image.  Row 0 is the top row.
+       Column 0 is the left column.
+    */
+    unsigned int row;
+    unsigned int col;
+} pm_pixelcoord;
+
 extern int pm_plain_output;
     /* Output functions are to produce plain (as opposed to raw) format
        regardless of their 'plainformat' arguments.
@@ -123,11 +131,11 @@ int
 pm_keymatch (char* const str, const char* const keyword, int const minchars);
 
 
-int 
-pm_maxvaltobits (int const maxval);
+int PURE_FN_ATTR
+pm_maxvaltobits(int const maxval);
 
-int 
-pm_bitstomaxval (int const bits);
+int PURE_FN_ATTR
+pm_bitstomaxval(int const bits);
 
 unsigned int PURE_FN_ATTR
 pm_lcm (unsigned int const x, 
@@ -152,7 +160,7 @@ pm_error (const char reason[], ...);
 void
 pm_perror (const char reason[]);           
 
-/* Obsolete - use shhopt and man page instead */
+/* Obsolete - use shhopt and user's manual instead */
 void 
 pm_usage (const char usage[]);             
 
