@@ -14,8 +14,6 @@
    - Change return type of asprintf to const char * and add strfree()
      to free that type.
 
-   - Define the NEED_* macros so we get all the routines.   
-
    - Change the name of the source file from snprintf.c to nstring.c
 
    - #include the interface header file instead of duplicating its
@@ -28,6 +26,11 @@
 
    - added stripeq().
 
+   - Define asprintf, etc. unconditionally.
+
+   - asprintf() returns strsol instead of NULL when can't get memory.
+     strfree() understands that.
+
  * AUTHOR
  *   Mark Martinec <mark.martinec@ijs.si>, April 1999.
  *
@@ -267,10 +270,6 @@
  * with "ap2 = ap", in this case you must not call va_end(ap2) !
  *   #define va_copy(ap2,ap) ap2 = ap
  */
-#define NEED_ASPRINTF   
-#define NEED_ASNPRINTF  
-#define NEED_VASPRINTF  
-#define NEED_VASNPRINTF 
 
 
 /* Define the following macros if desired:
@@ -903,107 +902,127 @@ snprintfN(char *       const dest,
 
 
 
-#if defined(NEED_ASPRINTF)
-int asprintfN(const char **ptr, const char *fmt, /*args*/ ...) {
-  va_list ap;
-  size_t str_m;
-  int str_l;
-  char *str;
+/* When a function that is supposed to return a malloc'ed string cannot
+   get the memory for it, it should return 'strsol'.  That has a much
+   better effect on the caller, if the caller doesn't explicitly allow for
+   the out of memory case, than returning NULL.  Note that it is very
+   rare for the system not to have enough memory to return a small string,
+   so it's OK to have somewhat nonsensical behavior when it happens.  We
+   just don't want catastrophic behavior.
 
-  va_start(ap, fmt);                            /* measure the required size */
-  str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);
-  va_end(ap);
-  assert(str_l >= 0);        /* possible integer overflow if str_m > INT_MAX */
-  str = (char *) malloc(str_m = (size_t)str_l + 1);
-  if (str == NULL) { errno = ENOMEM; str_l = -1; }
-  else {
-    int str_l2;
+   'strsol' is an external symbol, so if Caller wants to detect the
+   out-of-memory failure, he certainly can.
+*/
+const char * const strsol = "NO MEMORY TO CREATE STRING!";
+
+
+
+int
+vasprintfN(const char ** const ptr,
+           const char *  const fmt,
+           va_list             ap) {
+
+    size_t str_m;
+    int str_l;
+    char *str;
+    
+    str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);/*get required size*/
+    assert(str_l >= 0);     /* possible integer overflow if str_m > INT_MAX */
+    str_m = (size_t)str_l + 1;
+    str = malloc(str_m);
+    if (str == NULL) {
+        errno = ENOMEM;
+        str_l = -1;
+        *ptr = strsol;
+    } else {
+        int str_l2;
+        str_l2 = portable_vsnprintf(str, str_m, fmt, ap);
+        assert(str_l2 == str_l);
+        *ptr = str;
+    }
+    return str_l;
+}
+
+
+
+int
+asprintfN(const char ** const ptr,
+          const char *  const fmt, 
+          ...) {
+
+    int retval;
+    va_list ap;
+    
     va_start(ap, fmt);
-    str_l2 = portable_vsnprintf(str, str_m, fmt, ap);
+    
+    vasprintfN(ptr, fmt, ap);
+    
     va_end(ap);
-    assert(str_l2 == str_l);
-  }
-  *ptr = str;
-  return str_l;
+
+    return retval;
 }
-#endif
 
-#if defined(NEED_VASPRINTF)
-int vasprintfN(const char **ptr, const char *fmt, va_list ap) {
-  size_t str_m;
-  int str_l;
-  char *str;
 
-  str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);/*get required size*/
-  assert(str_l >= 0);        /* possible integer overflow if str_m > INT_MAX */
-  str = (char *) malloc(str_m = (size_t)str_l + 1);
-  if (str == NULL) { errno = ENOMEM; str_l = -1; }
-  else {
-    int str_l2 = portable_vsnprintf(str, str_m, fmt, ap);
-    assert(str_l2 == str_l);
-  }
-  *ptr = str;
-  return str_l;
-}
-#endif
 
-#if defined(NEED_ASNPRINTF)
-int asnprintfN (const char **ptr, size_t str_m, const char *fmt, /*args*/ ...) {
-  va_list ap;
-  int str_l;
-  char *str;
+int
+vasnprintfN(const char ** const ptr,
+            size_t        const str_m,
+            const char *  const fmt,
+            va_list             ap) {
 
-  va_start(ap, fmt);                            /* measure the required size */
-  str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);
-  va_end(ap);
-  assert(str_l >= 0);        /* possible integer overflow if str_m > INT_MAX */
-  if ((size_t)str_l + 1 < str_m) str_m = (size_t)str_l + 1;      /* truncate */
-  /* if str_m is 0, no buffer is allocated, just set str to NULL */
-  if (str_m == 0) {  /* not interested in resulting string, just return size */
-  } else {
-    str = (char *) malloc(str_m);
-    if (str == NULL) { errno = ENOMEM; str_l = -1; }
-    else {
-      int str_l2;
-      va_start(ap, fmt);
-      str_l2 = portable_vsnprintf(str, str_m, fmt, ap);
-      va_end(ap);
-      assert(str_l2 == str_l);
+    char *str;
+    int str_l;
+    
+    str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);/*get required size*/
+    assert(str_l >= 0);     /* possible integer overflow if str_m > INT_MAX */
+    if (str_m == 0) { 
+        /* not interested in resulting string, just return size */
+        *ptr = NULL;
+    } else {
+        size_t const truncatedSize = MIN((size_t)str_l + 1, str_m);
+        assert(truncatedSize > 0);
+        str = malloc(truncatedSize);
+        if (str == NULL) {
+            errno = ENOMEM;
+            str_l = -1;
+            *ptr = strsol;
+        } else {
+            int str_l2;
+            str_l2 = portable_vsnprintf(str, truncatedSize, fmt, ap);
+            assert(str_l2 == str_l); 
+            *ptr = str;
+        }
     }
-  }
-  *ptr = str;
-  return str_l;
+    return str_l;
 }
-#endif
 
-#if defined(NEED_VASNPRINTF)
-int vasnprintfN (const char **ptr, size_t str_m, const char *fmt, va_list ap) {
-  char *str;
-  int str_l;
 
-  str_l = portable_vsnprintf(NULL, (size_t)0, fmt, ap);/*get required size*/
-  assert(str_l >= 0);        /* possible integer overflow if str_m > INT_MAX */
-  if ((size_t)str_l + 1 < str_m) str_m = (size_t)str_l + 1;      /* truncate */
-  /* if str_m is 0, no buffer is allocated, just set str to NULL */
-  if (str_m == 0) {  /* not interested in resulting string, just return size */
-  } else {
-    str = (char *) malloc(str_m);
-    if (str == NULL) { errno = ENOMEM; str_l = -1; }
-    else {
-      int str_l2 = portable_vsnprintf(str, str_m, fmt, ap);
-      assert(str_l2 == str_l);
-    }
-  }
-  *ptr = str;
-  return str_l;
+
+int
+asnprintfN(const char ** const ptr,
+           size_t        const str_m,
+           const char *  const fmt,
+           ...) {
+
+    int retval;
+    va_list ap;
+
+    va_start(ap, fmt);                     /* measure the required size */
+    
+    retval = vasnprintfN(ptr, str_m, fmt, ap);
+    
+    va_end(ap);
+    
+    return retval;
 }
-#endif
 
 
 
 void
 strfree(const char * const string) {
-    free((void *) string);
+
+    if (string != strsol)
+        free((void *) string);
 }
 
 
