@@ -74,7 +74,7 @@ extern "C" {
    library subroutines whose names are the same except with the N at
    the end.  Because not all standard C libraries have them all,
    Netpbm must include them in its own libraries, and because some
-   standard C libraries have some of them, Neptbm must use different
+   standard C libraries have some of them, Netpbm must use different
    names for them.
    
    The GNU C library has all of them.  All but the oldest standard C libraries
@@ -85,7 +85,10 @@ extern "C" {
    The const is more correct.  
 
    strfree() is strictly a Netpbm invention, to allow proper type checking
-   when freeing storage allocated by the Netpbm asprintfN().  */
+   when freeing storage allocated by the Netpbm asprintfN().
+*/
+
+extern const char * const strsol;
 
 int
 snprintfN(char *       const dest,
